/*
 * AlgorithmPanel.java
 *
 * Created on 7. jen 2005, 14:58
 */

package gui;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Shape;
import java.util.ArrayList;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.JTextComponent;
import javax.swing.text.LayeredHighlighter;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;
import javax.swing.text.View;
import gui.utils.Codeline;


/**
 * Tento panel zobrazuje algoritmus a umoznuje zvyrazneni jedne jeho 
 * radky. Algoritmus se zadava jako pole retezcu, co radek to jedna
 * polozka v poli. Toto pole je pri prirazeni prevedeno na pole 
 * objektu {@link  gui.utils.Codeline }, ktere vytvori z kazdeho radku
 * dve kolekce,  kolekci slov a kolekci prirazenych stylu. Styly jsou 
 * definovany jako skonstanty v tride AlgorithmPanel.
 * @author Ondrej Hanousek
 */
public class AlgorithmPanel extends javax.swing.JPanel {
    /**
     * konstanta urcujici styl textu v algoritmu.
     */
    public static final String NORMAL="normal";
    /**
     * Konstanta urcujici styl klicovych slov.
     */
    public static final String KEYWORD="keyword";
    /**
     * Konstanta pro styl komentaru.
     */
    public static final String COMMENT="comment";
    
    
    private ArrayList<Integer> offsets=new ArrayList<Integer>();
    private DefaultHighlighter h=new DefaultHighlighter();
    
    /**
     * Vytvori prazdny panel pro algoritmus.
     */
    public AlgorithmPanel() {
        initComponents();
        text.setHighlighter(h);
    }
    
    /**
     * Zvyrazni zadany radek podbarvenim. Predchozi zvyrazneni jsou zrusena.
     * @param line cislo radku pro zvyrazneni. 
     */
    public void highlight(int line){
        h.removeAllHighlights();
        try {
            h.addHighlight(offsets.get(line-1), offsets.get(line), new BackgroundHighlightPainter(Color.yellow));
        } catch (BadLocationException ble) {
            System.err.println("Couldn't highlight text pane.");
        } catch (IndexOutOfBoundsException ine) {
            
        }
    }
    
    
    /**
     * Nastavi obsah panelu. Retezce jsou prvedeny na objekty 
     * {@link gui.utils.Codeline}, ktere zajisti zvyrazneni syntaxe.
     * @param codeline kazdy radek kodu je jednou polozkou v poli.
     */
    public void setText(String[] codeline){
        StyledDocument doc = text.getStyledDocument();
        addStylesToDocument(doc);
        
        try {
            int offsetCounter=0;
            offsets.add(0);
            for (int i=0; i<codeline.length; i++) {
                Codeline cl=new Codeline(codeline[i]);
                String[] words=cl.getParagraphs();
                String[] styles=cl.getStyles();
                for (int j=0;j<words.length;j++) {
                    doc.insertString(doc.getLength(), words[j], doc.getStyle(styles[j]));
                    offsetCounter+=words[j].length();
                }
                
                offsets.add(offsetCounter);
            }
        } catch (BadLocationException ble) {
            System.err.println("Couldn't insert initial text into text pane.");
        }
        
        
    }
    
    private void addStylesToDocument(StyledDocument doc) {
        
        Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
        Style regular = doc.addStyle(NORMAL, def);
        StyleConstants.setFontFamily(def, "SansSerif");
        Style s = doc.addStyle(COMMENT, regular);
        StyleConstants.setItalic(s, true);
        s = doc.addStyle(KEYWORD, regular);
        StyleConstants.setBold(s, true);
        
    }
    
    
    
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jScrollPane1 = new javax.swing.JScrollPane();
        text = new javax.swing.JTextPane();

        setLayout(new java.awt.BorderLayout());

        setMinimumSize(new java.awt.Dimension(200, 200));
        setPreferredSize(new java.awt.Dimension(300, 300));
        text.setEditable(false);
        text.setPreferredSize(new java.awt.Dimension(200, 200));
        jScrollPane1.setViewportView(text);

        add(jScrollPane1, java.awt.BorderLayout.CENTER);

    }
    // </editor-fold>//GEN-END:initComponents
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextPane text;
    // End of variables declaration//GEN-END:variables
    
    /**
     * Trida, ktera zajistuje podbarveni zvurazeneho radku.
     */
    public static class BackgroundHighlightPainter extends LayeredHighlighter.LayerPainter {
        private  Color color; // The color for the background
        /**
         * Vytvori zvyraznovac s definovanou barvou.
         * @param c Barva pozadi pro zvyrazneny radek.
         */
        public BackgroundHighlightPainter(Color c) {
            color = c;
        }
        
        /**
         * Neni nikdy volano.
         * @param g --
         * @param offs0 --
         * @param offs1 --
         * @param bounds --
         * @param c --
         */
        public void paint(Graphics g, int offs0, int offs1, Shape bounds,
                JTextComponent c) {
        }
        
        /**
         * Vykresli podbarvujici obdelnik.
         * @param g ..
         * @param offs0 ..
         * @param offs1 ..
         * @param bounds ..
         * @param c ..
         * @param view ..
         * @return ..
         */
        public Shape paintLayer(Graphics g, int offs0, int offs1, Shape bounds,
                JTextComponent c, View view) {
            g.setColor(color);
            Rectangle alloc=bounds.getBounds();
            g.fillRect(alloc.x,alloc.y, alloc.width,alloc.height);

            return alloc;
        }
        
        
    }
}
