/*
 * ArithmeticApplet.java
 *
 * Created on 26. listopad 2005, 14:33
 */

package gui;
import java.util.HashMap;
import java.util.Locale;
import javax.swing.JFrame;
import gui.parts.ARInterval;
import gui.parts.ARCharInfo;
import gui.utils.FibbCode;

/**
 * Trida pro vizualizaci aritmetickeho kodovani. Tato verze
 * pouziva aritmeticke kodovani v desetinne verzi. Proto je
 * pocet znaku ve vstupu doporuceno volit jako nasobek 10, coz
 * zajisti pomerne nazorne deleni.
 * @author Ondrej Hanousek
 */
public class ArithmeticApplet extends BasicApplet {
    
    private static int algNumLines=14;
    private static String[] lines;
    private int state=0;
    private int totalchars=0;
    private int position=0;
    private HashMap<String,ARCharInfo> characters=new HashMap<String,ARCharInfo>();
    private ARInterval I;
    private String X;
    private FibbCode len=new FibbCode(8);
    
    /**
     * Vytvori tridu pro vizualizaci aritmetickeho kodovani.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala systemove <PRE>Locales</PRE>. To zajistuje
     * zobrazeni popisku a textu v aplikaci ve spravnem jazyce,
     * pokud je k dispozici.
     */
    public ArithmeticApplet() {
    }
    
    /**
     * Vytvori tridu pro vizualizaci aritmetickeho kodovani.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala <PRE>Locales</PRE> zadane parametrem. To
     * zajistuje zobrazeni popisku a textu v aplikaci ve spravnem
     * jazyce, pokud je k dispozici.
     * @param l <PRE>Locales </PRE> nastavene pro aplikaci
     */
    public ArithmeticApplet(Locale l) {
        Localisation.setLocale(l);
       
    }
    
    /**
     * Tato funkce se vola pri inicializaci appletu prohlizecem.
     * Take je volana z metody main pri spusteni jako aplikace.
     * V teto metode se naplni prislusny {@link gui.AlgorithmPanel}
     * lokalizovanou ({@link gui.Localisation }) verzi algoritmu.
     */
    public void init(){
        
        try {
            Localisation.setLocale(new Locale(getParameter("locale")));
        } catch (NullPointerException e) {
            //  Pouziji se defaultni, zde nic nesmi byt, aby to fungovalo i jako aplikace.
        }
        super.init();
        setCodelines(getCodelines());
        setHeadLine(Localisation.getString("arHeadline"));
        highlightLine(1);
         hideStatsPanel();
    }
    
    private String[] getCodelines(){
        if (lines==null) fillAlgLines();
        return lines;
    }
    
    private void fillAlgLines(){
        lines=new String[algNumLines];
        for (int i=0;i<algNumLines;i++){
            String pom=Localisation.getString("arAlgLine"+(i+1));
            lines[i]=pom;
        }
    }
    
    /**
     * Pretizeni abstraktni metody ze tridy {@link gui.BasicApplet },
     * ktera je volana pri kazdem kroku - tj. po akci tlacitka
     * "krok" nebo pravidelne ve stavu "spusteno".
     * @return 0 - v pripade ze jsou mozne dalsi kroky,
     * -1 - kdyz neni dalsi krok mozny.
     */
    public int doNextStep() {
        switch (state) {
            //<editor-fold defaultstate="collapsed" desc=" State==0 - cetnosti " >
            case 0:
                highlightLine(2);
                String pom=readNext();
                if (pom.compareTo("")==0) {
                    state=99;
                    break;
                }
                totalchars++;
                ARCharInfo tempInfo=new ARCharInfo(pom);
                
                if (characters.containsKey(pom)) {
                    characters.get(pom).incrementCounter();
                } else {
                    characters.put(pom,tempInfo);
                    tempInfo.setPosition(position++);
                    addPart(tempInfo);
                }
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==1 - Interval I " >
            case 1:
                highlightLine(4);
                I=new ARInterval(0f,1f);
                I.moveTo(80,ARInterval.Y_OFFSET);
                addPart(I);
                state=2;
                break;
                
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==2 - rozdeleni I " >
            case 2:
                highlightLine(5);
                I.setUnits(characters.values());
                state=3;
                break;
                
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==3 - Nacti X " >
            case 3:
                rewindInput();
                X=readNext();
                highlightLine(6);
                state=4;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==4 - Podminka while" >
            case 4:
                highlightLine(7);
                if (X!="") {
                    state=5;
                } else {
                    state=8;
                }
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==5 - Novy interval" >
            case 5:
                highlightLine(9);
                ARInterval I2=new ARInterval(characters.get(X).getFrom(), characters.get(X).getTo());
                I2.moveTo(I.getBounds().width+I.getBounds().x+ARInterval.X_SPACING, ARInterval.Y_OFFSET);
                I.setNext(I2);
                addPart(I2);
                I.setHighlighted(false);
                I=I2;
                state=6;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==6 - Deleni intervalu" >
            case 6:
                highlightLine(10);
                I.setUnits(characters.values());
                state=7;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==7 - Dalsi znak" >
            case 7:
                highlightLine(11);
                X=readNext();
                state=4;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==8 - Vypis vystupu" >
            case 8:
                highlightLine(13);
                write(Double.toString(I.getFrom()+(I.getTo()-I.getFrom())/2));
                state=9;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==9 - Konec" >
            case 9:
                write("");
                highlightLine(14);
                repaint();
                return -1;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==99 - Vypis cetnosti" >
            case 99:
                highlightLine(3);
                for (char c : VALID.toCharArray()) {
                    String key=(new Character(c)).toString();
                    String out=characters.containsKey(key)?len.fibbCode(characters.get(key).getCount()):len.fibbCode(0);
                    write(out+" ");
                    addOutBits(out.length());
                }
                write("");
                repaint();
                state=1;
                break;
                // </editor-fold>
                
        }
        repaint();
        return 0;
    }
    
    /**
     * Metoda volana pri spousteni jako aplikace.
     * @param args parametr muze byt ve tvaru <PRE>Locale</PRE> pro nastaveni jineho jazyka.
     */
    public static void main(String[] args) {
        ArithmeticApplet applet;
        if (args.length==1) {
            Localisation.setLocale(new Locale(args[0]));
        }
        applet = new ArithmeticApplet();
        JFrame window = new JFrame(Localisation.getString("arTitle"));
        window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        window.setContentPane(applet);
        applet.init();
        window.pack();
        window.setVisible(true);
    }
    
    /**
     * Zavolano z tridy {@link gui.BasicApplet }, pokud se ma obnovit puvodni
     * nastaveni appletu.
     */
    public void reset() {
        characters=new HashMap<String,ARCharInfo>();
        position=0;
        super.reset();
        state=0;
    }
    
    
    
}
