/*
 * BasicApplet.java
 *
 * Created on 16. jen 2005, 10:27
 */

package gui;
import java.io.File;
import java.security.AccessControlException;
import java.util.ResourceBundle;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import gui.utils.EpsFileFilter;
import gui.utils.PngFileFilter;
import gui.utils.Steped;
import gui.utils.Stepper;
import gui.parts.Part;
import gui.utils.BwEpsFileFilter;
import gui.utils.ColorEpsFileFilter;
import gui.utils.GrEpsFileFilter;
import org.jibble.epsgraphics.EpsGraphics2D;


/**
 * Abstraktni trida, ktera vytvari GUI a definuje spolecne akce. Z teto 
 * tridy jsou zdedeny tridy, ktere implementuji vizualizaci jednotlivych
 * algoritmu.
 * @author Ondrej Hanousek
 */
public abstract class BasicApplet extends javax.swing.JApplet implements Steped{
    
    /**
     * Retezec s abecedou - platnymi znaky.
     */
    public static final String VALID="abcdefg";
    private boolean running=false;
    private Stepper s;
    private JPanel additionalControls;
    
    /**
     * Pri volani konstruktoru se vytvari propojeni s rohranim pro krokovani.
     */
    public BasicApplet(){
        s=new Stepper(this);
    }
    
    private String lastDir=System.getProperty("user.home");
    
    
    /**
     * Inicializace - pri spusteni jako applet se vola automaticky, jinak musi
     * byt voana v konstruktoru implementujici metody.
     */
    public void init(){
        initComponents();
        splitPane.setDividerLocation(0.5);
        statsPanel1.setInBits(inputPanel1.getText().length()*charSize());
    }
    
    /**
     * Spocita velikost blokoveho kodu znaku v bitech, vypocita se z retezce VALID.
     * @return delka blokoveho kodu znaku
     */
    public static int charSize() {
        return ((int) Math.ceil(Math.log(VALID.length())/Math.log(2.0)));
    }
    
    /**
     * Tato metoda je volana pri zadani noveho vstupu do dialogu a 
     * proveri jeho platnost. Porovnani s abecedou (retezcem VALID) 
     * je provedeno primo v dialogu. Toto slouzi pro jemnejsi rozliseni
     * neplatnych retezcu. Dokud vraci false, neni dialogove okno uzavreno.
     * @param s retezec k overeni
     * @return povoleny retezec
     */
    public boolean isValidInput(String s){
        return true;
    }
    
    /**
     * Pokud je retezec neplatny po volani metody <PRE>isValidInputString</PRE>,
     * je volana tato metoda. Pri pretizeni by mela vracet lokalizovanou
     * chybovou zpravu.
     * @return Lokalizovana zprava o chybnem vstupnim retezci.
     */
    public String getInvalidInputMsg() {
        return "";
    }
    
    /**
     * Obnivi vychozi stav GUI.
     */
    public void reset() {
        s.interrupt();
        inputPanel1.rewind();
        alg.highlight(1);
        drawPanel1.reset();
        outputPanel1.reset();
        statsPanel1.setInBits(inputPanel1.getText().length()*charSize());
        statsPanel1.resetOutBits();
        repaint();
    }
    
    /**
     * Tato metoda je volana, kdyz neni zadny dalsi krok mozny. 
     * Upravi gui tak, aby to bylo patrne.
     */
    public void steppingEnded(){
        stepBtn.setEnabled(false);
        entryBtn.setEnabled(true);
        resetBtn.setEnabled(true);
        saveBtn.setEnabled(true);
        runBtn.setEnabled(false);
        runBtn.setText(Localisation.getString("run"));
        running=false;
    }
    
    
   private void save(){
        try {
            JFileChooser save=new JFileChooser(lastDir);
            save.addChoosableFileFilter(new PngFileFilter());
            save.addChoosableFileFilter(new ColorEpsFileFilter());
            save.addChoosableFileFilter(new GrEpsFileFilter());
            save.addChoosableFileFilter(new BwEpsFileFilter());
            save.setDialogTitle(Localisation.getString("saveImg"));
            save.setLocale(Localisation.getLocale());
            int result=save.showSaveDialog(this);
            if (result==JFileChooser.APPROVE_OPTION) {
                int colorMode=EpsGraphics2D.RGB;
                File outputFile=save.getSelectedFile();
                if (!(outputFile.getName().endsWith(".png") || outputFile.getName().endsWith(".eps"))) {
                    if ( save.getFileFilter() instanceof EpsFileFilter) {
                        outputFile=new File(outputFile.getAbsolutePath()+".eps");
                        colorMode=((EpsFileFilter) save.getFileFilter()).getColorMode();
                    } else {
                        outputFile=new File(outputFile.getAbsolutePath()+".png");
                    }
                }
                if (outputFile.getName().endsWith(".png")) {
                    drawPanel1.writePngImage(outputFile);
                } else {
                    //     System.out.println(colorMode);
                    drawPanel1.writePsImage(outputFile,colorMode);
                }
                lastDir=outputFile.getParent();
            }
        } catch (AccessControlException ex) {
            
            JOptionPane.showMessageDialog(this, Localisation.getString("accessError") ,Localisation.getString("accessErrorTitle"), JOptionPane.ERROR_MESSAGE);
        }
        
    }
    
    /**
     * Volanim teto metody se nastavi radky algoritmu do panelu AlgorithmPanel.
     * @param codelines Radky algoritmu, co radek to polozka v poli.
     */
    public void setCodelines(String[] codelines){
        alg.setText(codelines);
    }
    
    /**
     * Volanim teto metody se nastavi prvni radek - titulek v appletu/aplikaci.
     * @param s retezec, ktery ma byt nastaven na prvni radek
     */
    public void setHeadLine(String s){
        headline.setText(s);
    }
    
    /**
     * Testuje, zda je konec vstupniho souboru.
     * @return <PRE>true</PRE> pokud je konec vstupu
     */
    public boolean eof(){
        return inputPanel1.eof();
    }
    
    /**
     * Vraci dalsi znak ze vstupu a oznaci ho. Pokud neni zadny dalsi znak ve vstupu, 
     * vrati "" a oznaceni zrusi.
     * @return dalsi znak nebo "" pri konci vstupu
     */
    public String readNext(){
        return inputPanel1.readNext();
    }
    
    /**
     * Nastavi vstup opet na zacatek a zrusi oznaceni.
     */
    public void rewindInput(){
        inputPanel1.rewind();
    }
    
    /**
     * Zapise do vyspupu retezec a zvyrazni ho.
     * @param s retezec do vystupu
     */
    public void write(String s){
        outputPanel1.write(s);
    }
    
    /**
     * Podbarvenim zvyrazni pozadovnaou radku v kodu.
     * @param line radka, ktera ma byt zvyraznena
     */
    public void highlightLine(int line){
        alg.highlight(line);
    }
    
    /**
     * Prida objekt trida {@link gui.parts.Part } do panelu s grafikou. Tento objekt je pak vykreslovan.
     * @param p graficky objekt, ktery ma byt pridan k vykreslovanym
     */
    public void addPart(Part p){
        drawPanel1.addPart(p);
    }
    
    /**
     * Odebere objekt tridy  {@link gui.parts.Part } z kolekce vykreslovanych objektu.
     * @param p objekt ktery jiz nema byt vykreslovan.
     */
    public void removePart(Part p){
        drawPanel1.removePart(p);
    }
    
    /**
     * Nastavi pridavne ovladaci prvky do panelu pod zakladnimi ovladacimi prvky.
     * Muze byt pouzito, pokud vizualizace umoznuje zmenu parametru algoritmu.
     * @param panel panel s ovladacimi prvky
     */
    public void setAdditionalControlsPanel(JPanel panel){
        additionalControls=panel;
    }
    
    private void newEntry() {
        reset();
        EntryDialog ed=new EntryDialog(this,inputPanel1.getText(), VALID);
        ed.setModal(true);
        ed.setVisible(true);
        if (!ed.canceled()) {
            inputPanel1.setText(ed.getText());
            inputPanel1.rewind();
            stepBtn.setEnabled(true);
            runBtn.setEnabled(true);
            resetBtn.setEnabled(true);
            saveBtn.setEnabled(true);
        }
        reset();
    }
    
    
    /**
     * Zvysi citac vystupnich bitu.
     * @param count pocet bitu ke zvyseni
     */
    public void addOutBits(int count){
        statsPanel1.addOutBits1(count);
    }
    
    /** This method is called from within the init() method to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        headline = new javax.swing.JLabel();
        controls = new javax.swing.JPanel();
        stepBtn = new javax.swing.JButton();
        runBtn = new javax.swing.JButton();
        entryBtn = new javax.swing.JButton();
        resetBtn = new javax.swing.JButton();
        saveBtn = new javax.swing.JButton();
        inout = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        inputPanel1 = new gui.InputPanel();
        outputPanel1 = new gui.OutputPanel();
        splitPane = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        drawPanel1 = new gui.DrawPanel();
        alg = new gui.AlgorithmPanel();
        additional = additionalControls!=null?additionalControls:new JPanel();
        statsPanel1 = new gui.StatsPanel();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        });

        headline.setFont(new java.awt.Font("Tahoma", 1, 24));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(headline, gridBagConstraints);

        controls.setLayout(new java.awt.GridLayout(1, 0));

        stepBtn.setText(Localisation.getString("step"));
        stepBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stepBtnActionPerformed(evt);
            }
        });

        controls.add(stepBtn);

        runBtn.setText(Localisation.getString("run"));
        runBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                runBtnActionPerformed(evt);
            }
        });

        controls.add(runBtn);

        entryBtn.setText(Localisation.getString("newIn"));
        entryBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                entryBtnActionPerformed(evt);
            }
        });

        controls.add(entryBtn);
        entryBtn.getAccessibleContext().setAccessibleName(java.util.ResourceBundle.getBundle("gui/Locales").getString("newIn"));

        resetBtn.setText(Localisation.getString("reset"));
        resetBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resetBtnActionPerformed(evt);
            }
        });

        controls.add(resetBtn);

        saveBtn.setText(Localisation.getString("saveImage"));
        saveBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveBtnActionPerformed(evt);
            }
        });

        controls.add(saveBtn);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(controls, gridBagConstraints);

        inout.setLayout(new java.awt.GridBagLayout());

        jLabel3.setFont(new java.awt.Font("Dialog", 0, 14));
        jLabel3.setText(Localisation.getString("input"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHEAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        inout.add(jLabel3, gridBagConstraints);

        jLabel4.setFont(new java.awt.Font("Dialog", 0, 14));
        jLabel4.setText(Localisation.getString("output"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        inout.add(jLabel4, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        inout.add(inputPanel1, gridBagConstraints);

        outputPanel1.setMinimumSize(new java.awt.Dimension(56, 45));
        outputPanel1.setPreferredSize(new java.awt.Dimension(56, 45));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        inout.add(outputPanel1, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(inout, gridBagConstraints);

        jScrollPane1.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        jScrollPane1.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        jScrollPane1.setMaximumSize(new java.awt.Dimension(3000, 32767));
        jScrollPane1.setMinimumSize(new java.awt.Dimension(250, 200));
        jScrollPane1.setOpaque(false);
        jScrollPane1.setPreferredSize(new java.awt.Dimension(300, 200));
        jScrollPane1.setViewportView(drawPanel1);

        splitPane.setLeftComponent(jScrollPane1);

        splitPane.setRightComponent(alg);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 5, 5);
        getContentPane().add(splitPane, gridBagConstraints);

        additional.setMinimumSize(new java.awt.Dimension(0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        getContentPane().add(additional, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        getContentPane().add(statsPanel1, gridBagConstraints);

    }
    // </editor-fold>//GEN-END:initComponents
    
    private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
        splitPane.setDividerLocation(0.5);
    }//GEN-LAST:event_formComponentResized
    
    private void stepBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stepBtnActionPerformed
       if (doNextStep()==-1) steppingEnded();
    }//GEN-LAST:event_stepBtnActionPerformed
    
    private void runBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_runBtnActionPerformed
        if (!running) {
            runBtn.setText("Stop");
            stepBtn.setEnabled(false);
            entryBtn.setEnabled(false);
            resetBtn.setEnabled(false);
            saveBtn.setEnabled(false);
            
            s=new Stepper(500,this);
            s.start();
            running=true;
        } else {
            s.endStepping();
            stepBtn.setEnabled(true);
            entryBtn.setEnabled(true);
            resetBtn.setEnabled(true);
            saveBtn.setEnabled(true);
            runBtn.setText(ResourceBundle.getBundle("gui/Locales",getLocale()).getString("run"));
            running=false;
        }
    }//GEN-LAST:event_runBtnActionPerformed
    
    private void resetBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resetBtnActionPerformed
        reset();
        stepBtn.setEnabled(true);
        runBtn.setEnabled(true);
        resetBtn.setEnabled(true);
        saveBtn.setEnabled(true);
    }//GEN-LAST:event_resetBtnActionPerformed
    
    private void entryBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_entryBtnActionPerformed
        newEntry();
    }//GEN-LAST:event_entryBtnActionPerformed
    
    private void saveBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveBtnActionPerformed
        save();
    }//GEN-LAST:event_saveBtnActionPerformed
    
    /**
     * Skryje panel se statistikami o kompresnim pomeru. Pouziva se u 
     * aritmetickoho kodovani
     */
    public void hideStatsPanel(){
        statsPanel1.setVisible(false);
    }
    
    
    public void addOutBits2(int bits){
        statsPanel1.addOutBits2(bits);
    }
    
    public void setTwoCounters(boolean b){
        statsPanel1.setTwoCounters(b);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel additional;
    private gui.AlgorithmPanel alg;
    private javax.swing.JPanel controls;
    private gui.DrawPanel drawPanel1;
    private javax.swing.JButton entryBtn;
    private javax.swing.JLabel headline;
    private javax.swing.JPanel inout;
    private gui.InputPanel inputPanel1;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JScrollPane jScrollPane1;
    private gui.OutputPanel outputPanel1;
    private javax.swing.JButton resetBtn;
    private javax.swing.JButton runBtn;
    private javax.swing.JButton saveBtn;
    private javax.swing.JSplitPane splitPane;
    private gui.StatsPanel statsPanel1;
    private javax.swing.JButton stepBtn;
    // End of variables declaration//GEN-END:variables
    
}
