/*
 * FGKApplet.java
 *
 * Created on 6. listopad 2005, 22:01
 *
 */

package gui;
import gui.parts.Legend;
import java.util.HashMap;
import java.util.Locale;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.swing.JFrame;
import gui.parts.FGKCharInfo;


/**
 * Vizualizace adaptivniho Huffmanova kodovani s jadrem FGK.
 * @author Ondrej Hanousek
 */
public class FGKApplet extends BasicApplet {
    private static int algNumLines=30;
    private static String[] lines;
    private int state;
    private TreeSet<FGKCharInfo> allNodes=new TreeSet<FGKCharInfo>();
    private HashMap<String,FGKCharInfo> charNodes=new HashMap<String,FGKCharInfo>();
    private FGKCharInfo zero,root,U,tmp;
    private  String X="";
    private Legend orderLegend=new Legend(FGKCharInfo.ORDER_COLOR,Localisation.getString("fgkOrderLegend"));
    private Legend codeLegend=new Legend(FGKCharInfo.CODE_COLOR,Localisation.getString("fgkCodeLegend"));    
    private Legend countLegend=new Legend(FGKCharInfo.COUNT_COLOR,Localisation.getString("fgkCountLegend"));
   private Legend selectedLegend=new Legend(FGKCharInfo.SELECTED_COLOR, Localisation.getString("fgkSelectedLegend"));
    
    /**
     * Vytvori tridu pro vizualizaci dynamickeho Huffmanova kodovani.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala systemove <PRE>Locales</PRE>. To zajistuje
     * zobrazeni popisku a textu v aplikaci ve spravnem jazyce,
     * pokud je k dispozici.
     */
    public FGKApplet() {
    }
    
    /**
     * Vytvori tridu pro vizualizaci dynamickeho Huffmanova kodovani.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala <PRE>Locales</PRE> zadane parametrem. To
     * zajistuje zobrazeni popisku a textu v aplikaci ve spravnem
     * jazyce, pokud je k dispozici.
     * @param l <PRE>Locales </PRE> nastavene pro aplikaci
     */
    public FGKApplet(Locale l) {
        Localisation.setLocale(l);
        
    }

       
    private void fillAlgLines(){
        lines=new String[algNumLines];
        for (int i=0;i<algNumLines;i++){
            String pom=Localisation.getString("fgkAlgLine"+(i+1));
            lines[i]=pom;
        }
    }
    
    
    /**
     * Tato funkce se vola pri inicializaci appletu prohlizecem.
     * Take je volana z metody main pri spusteni jako aplikace.
     * V teto metode se naplni prislusny {@link gui.AlgorithmPanel}
     * lokalizovanou ({@link gui.Localisation }) verzi algoritmu.
     */
    public void init(){
        
        try {
            Localisation.setLocale(new Locale(getParameter("locale")));
        } catch (NullPointerException e) {
            //  Pouziji se defaultni, zde nic nesmi byt, aby to fungovalo i jako aplikace.
        }
        super.init();
        setCodelines(getCodelines());
        setHeadLine(Localisation.getString("fgkHeadline"));
        highlightLine(1);
    }

    
    /**
     * Metoda volana pri spousteni jako aplikace.
     * @param args parametr muze byt ve tvaru <PRE>Locale</PRE> pro nastaveni jineho jazyka.
     */
    public static void main(String[] args) {
        FGKApplet applet;
       if (args.length==1) {
            Localisation.setLocale(new Locale(args[0]));
        }
            applet = new FGKApplet();
        
        JFrame window = new JFrame(Localisation.getString("fgkTitle"));
        window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        window.setContentPane(applet);
        applet.init();
        window.pack();
        window.setVisible(true);
    }
    
    private String[] getCodelines(){
        if (lines==null) fillAlgLines();
        return lines;
    }
    
    /**
     * Pretizeni abstraktni metody ze tridy {@link gui.BasicApplet },
     * ktera je volana pri kazdem kroku - tj. po akci tlacitka
     * "krok" nebo pravidelne ve stavu "spusteno".
     * @return 0 - v pripade ze jsou mozne dalsi kroky,
     * -1 - kdyz neni dalsi krok mozny.
     */
    public int doNextStep() {
        
        switch (state) {
            //<editor-fold defaultstate="collapsed" desc=" State==0 - uzel ZERO, legenda " >
            case 0:
                highlightLine(2);
                state=1;
                root=zero=new FGKCharInfo();
                zero.setOrder(100);
                zero.setCount(0);
                allNodes.add(zero);
                addPart(zero);
                countLegend.moveTo(75,0);
                codeLegend.moveTo(75,10);
                orderLegend.moveTo(75,20);
                selectedLegend.moveTo(75,30);
                addPart(countLegend);
                addPart(codeLegend);
                addPart(orderLegend);
                addPart(selectedLegend);
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==1 - nacti prvni znak " >
            case 1:
                highlightLine(3);
                state=2;
                X=readNext();
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==2 - Podminka while " >
            case 2:
                highlightLine(4);
                
                if (X.compareTo("")!=0) {
                    state=3;
                } else {
                    state=20;
                }
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==3 - Kontrola znaku " >
            case 3:
                highlightLine(5);
                if (!charNodes.containsKey(X)) {
                    state=4;
                } else {
                    state=17;
                }
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==4 -Vypis kodu zero" >
            case 4:
                highlightLine(7);
                state=5;
                write(zero.getCode());
                addOutBits(zero.getCode().length());
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==5 -Vypis znaku na vystup" >
            case 5:
                highlightLine(8);
                state=6;
                write(X);
                addOutBits(charSize());
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==6 -Rozdvojeni uzlu zero" >
            case 6:
                highlightLine(9);
                state=7;
                write("");
                FGKCharInfo oldP=zero.getParent();
                FGKCharInfo newByte=new FGKCharInfo(X);
                FGKCharInfo newZero=new FGKCharInfo();
                
                int order=zero.getOrder();
                newByte.setOrder(order-1);
                newZero.setOrder(order-2);
                
                zero.setOne(newByte);
                zero.setZero(newZero);
                U=zero;
                
                allNodes.add(newByte);
                allNodes.add(newZero);
                charNodes.put(X,newByte);
                
                addPart(newByte);
                addPart(newZero);
                
                zero=newZero;
                
                
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==7 -volani aktualizuj strom" >
            case 7:
                U.setHighlighted(true);
                highlightLine(10);
                state=8;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==8 - procedura aktualizuj_stromu" >
            case 8:
                highlightLine(21);
                state=9;
                U.setHighlighted(true);
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==9 - AS - podminka ve while" >
            case 9:
                highlightLine(23);
                if (U.isRoot()) {
                    state=14;
                } else {
                    state=10;
                }
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==10 - AS - bude vymena?" >
            case 10:
                highlightLine(24);
                SortedSet<FGKCharInfo> pom=allNodes.tailSet(U);
                tmp=U;
                int count=U.getCount();
                for (FGKCharInfo i:pom){
                    if (i.getCount()==count) {
                        tmp=i;
                    } else {
                        break;
                    }   
                }
             
                if (tmp!=U && tmp!=U.getParent()) { //probehne vymena
                    tmp.setHighlighted(true);
                    state=11;
                } else {
                    state=12;
                }
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==11 - AS - vymena " >
            case 11:
                highlightLine(25);
                state=12;
                
                //       System.out.println(allNodes);
                
                FGKCharInfo p1=tmp.getParent();
                FGKCharInfo p2=U.getParent();
                
                p1.changeChildren(tmp,U);
                p2.changeChildren(U,tmp);
                
                allNodes.remove(tmp);
                allNodes.remove(U);
                
                int o=tmp.getOrder();
                
                tmp.setOrder(U.getOrder());
                U.setOrder(o);
                
                allNodes.add(tmp);
                allNodes.add(U);
                
                tmp.setParent(p2);
                U.setParent(p1);
                
                root.updatePositions();
                //p2.updatePositions();
                
                //    System.out.println(allNodes);
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==12 - AS - zvyseni poctu" >
            case 12:
                highlightLine(26);
                state=13;
                tmp.setHighlighted(false);
                U.setHighlighted(true);
                U.incrementCounter();
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==12 - AS - U=parent U" >
            case 13:
                highlightLine(27);
                state=9;
                U.setHighlighted(false);
                
                U=U.getParent();
                U.setHighlighted(true);
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==13 - AS - root ++" >
            case 14:
                highlightLine(29);
                state=19;
                root.incrementCounter();
                root.updateChildrenCodes();
                U.setHighlighted(false);
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==17 -vypis kodu X" >
            case 17:
                highlightLine(14);
                state=18;
                write(charNodes.get(X).getCode());
                addOutBits(charNodes.get(X).getCode().length());
                U=charNodes.get(X);
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==18 -volani aktualizace Stromu" >
            case 18:
                highlightLine(15);
                state=8;
                write("");
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==19 -dalsi znak" >
            case 19:
                highlightLine(17);
                state=2;
                X=readNext();
                
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==20 -end" >
            case 20:
                highlightLine(19);
                return -1;
                // </editor-fold>
        }
        repaint();
        return 0;
    }
    
    
    /**
     * Zavolano z tridy {@link gui.BasicApplet }, pokud se ma obnovit puvodni
     * nastaveni appletu.
     */
    public void reset(){
        allNodes=new TreeSet<FGKCharInfo>();
        charNodes=new HashMap<String,FGKCharInfo>();
        state=0;
        super.reset();
    }
    
    
}
