/*
 * InputPanel.java
 *
 * Created on 4. jen 2005, 9:48
 */

package gui;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Rectangle;
import javax.swing.JFrame;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter;
import gui.utils.BoxHighlightPainter;

/**
 * Trida, ktera se stara o panel se vstupnim textem, vcetne 
 * zvyrazneni posledne nacteneho znaku.
 * @author Ondrej Hanousek
 */
public class InputPanel extends javax.swing.JPanel {
    private  String text="abcdefafaaacaa";
    private int position=-1;
    private DefaultHighlighter h=new DefaultHighlighter();
    
    /**
     * Vytvori novy panel s prazdnym vstupem.
     */
    public InputPanel(){
        initComponents();
        line.setHighlighter(h);
    }
    
    
    /**
     * Nastavi novy vstupni text.
     * @param text vstupni text
     */
    public void setText(String text){

        this.text=text;
        line.setText(text);
        rewind();
    }
    
    /**
     * Nacte dalsi znak ze vstupu, oznaci ho.
     * @return znak ze vstupu, prazdny retezec pokud neni 
     * zadny dalsi znak.
     */
    public String readNext(){
        h.removeAllHighlights();
        if (position<text.length()-1) {
            position++;
            FontMetrics m=line.getFontMetrics(line.getFont());
            int x=m.charsWidth(text.toCharArray(), 0, position);
            int y=0;
            int width=m.charWidth(text.charAt(position));
            int height=m.getHeight();
            Rectangle bounds=new Rectangle(x, y, width, height);
            try {
                h.addHighlight(0, 1, new BoxHighlightPainter(Color.red, bounds));
            } catch (BadLocationException ex) {
                System.err.println(ex.getLocalizedMessage());
            }
            
            Rectangle v=getVisibleRect();
            
            Rectangle r=new Rectangle(bounds.x-v.width/2,0,v.width,v.height);
            line.scrollRectToVisible(r);
            return text.substring(position,position+1);
        }
        return "";
    }
    
    /**
     * Metoda pro ziskani celeho vstupniho retezce. 
     * Nema vliv na zvyrazneni.
     * @return cely vstupni reezec
     */
    public String getText(){
        return text;
    }
    
    /**
     * Metoda pro kontrolu, zda je ve vstupu jeste nejaky znak.
     * @return <PRE>true</PRE> - kdyz vstup je jiz cely precten
     * <PRE>false</CODE> - jinak
     */
    public boolean eof(){
        return (position>=text.length()-1);
    }
    
    /**
     * Odstrani zvyrazneni ze vstupniho retezce.
     */
    public void removeHighLights(){
        h.removeAllHighlights();
    }
    
    /**
     * Vrati se na zacatek vstupniho retezce.
     */
    public void rewind(){
        this.position=-1;
        h.removeAllHighlights();
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jScrollPane1 = new javax.swing.JScrollPane();
        panel = new javax.swing.JPanel();
        line = new javax.swing.JTextArea();

        setLayout(new java.awt.GridBagLayout());

        setMinimumSize(new java.awt.Dimension(56, 45));
        setPreferredSize(new java.awt.Dimension(56, 45));
        jScrollPane1.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        jScrollPane1.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        panel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 0, 0));

        line.setBackground(panel.getBackground());
        line.setEditable(false);
        line.setFont(new java.awt.Font("Dialog", 1, 18));
        line.setRows(1);
        line.setText(getText());
        panel.add(line);

        jScrollPane1.setViewportView(panel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(jScrollPane1, gridBagConstraints);

    }
    // </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextArea line;
    private javax.swing.JPanel panel;
    // End of variables declaration//GEN-END:variables
    
 
}
