package gui;
import gui.utils.FibbCode;
import java.util.Locale;
import javax.swing.JFrame;
import gui.parts.Lz77Window;

/**
 * Vytvori novy panel s ovladanim parametru LZ 77.
 * @author Ondrej Hanousek
 */
public class Lz77Applet extends BasicApplet {
    
    private static int algNumLines=11;
    private static String[] lines;
    private Lz77Panel sizePanel=new Lz77Panel();
    private Lz77Window window=new Lz77Window(0,0);
    private int state=0;
    private FibbCode len=new FibbCode(3);
    
    /**
     * Vytvori tridu pro vizualizaci kodovani LZ 77.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala systemove <PRE>Locales</PRE>. To zajistuje
     * zobrazeni popisku a textu v aplikaci ve spravnem jazyce,
     * pokud je k dispozici.
     */
    public Lz77Applet() {
        setAdditionalControlsPanel(sizePanel);
    }
    
    /**
     * Vytvori tridu pro vizualizaci kodovani LZ 77.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala <PRE>Locales</PRE> zadane parametrem. To
     * zajistuje zobrazeni popisku a textu v aplikaci ve spravnem
     * jazyce, pokud je k dispozici.
     * @param l <PRE>Locales </PRE> nastavene pro aplikaci
     */
    public Lz77Applet(Locale l) {
        Localisation.setLocale(l);
        setAdditionalControlsPanel(sizePanel);
    }
    
    
    
    /**
     * Zavolano z tridy {@link gui.BasicApplet }, pokud se ma obnovit puvodni
     * nastaveni appletu.
     */
    public void reset(){
        super.reset();
        state=0;
        sizePanel.setEnabled(true);
    }
    
    /**
     * Pretizeni abstraktni metody ze tridy {@link gui.BasicApplet },
     * ktera je volana pri kazdem kroku - tj. po akci tlacitka
     * "krok" nebo pravidelne ve stavu "spusteno".
     * @return 0 - v pripade ze jsou mozne dalsi kroky,
     * -1 - kdyz neni dalsi krok mozny.
     */
    public int doNextStep() {
        switch (state) {
            //<editor-fold defaultstate="collapsed" desc=" State==0 - inicializace " >
            case 0:
                highlightLine(1);
                sizePanel.setEnabled(false);
                window=new Lz77Window(sizePanel.getCodedSize(), sizePanel.getViewSize());
                addPart(window);
                state=1;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==1 - naplneni"   >
                
            case 1:
                highlightLine(2);
                for (int i=0;i<sizePanel.getViewSize();i++) {
                    String pom=readNext();
                    if (pom=="") pom=" ";
                    window.addData(pom);
                }
                state=2;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==2 - podminka while">
                
            case 2:
                highlightLine(3);
                if (window.isViewEmpty())  {
                    state=9;
                    break;
                }
                state=3;
                break;
// </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==3 - hledani p">
            case 3:
                highlightLine(4);
                window.findPrefix();
                state=4;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==4 - i ">
            case 4:
                highlightLine(5);
                window.showI(true);
                state=5;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==5 - j ">
            case 5:
                highlightLine(6);
                window.showJ(true);
                state=6;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==6 - x ">
            case 6:
                highlightLine(7);
                window.showX(true);
                state=7;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==7 - vypis">
            case 7:
                highlightLine(8);
                state=8;
                window.showI(false);
                window.showJ(false);
                window.showX(false);
                window.showPrefix(false);
                write(window.getOutput());
                addOutBits(len.fibbLength(window.getI()));
                addOutBits(len.fibbLength(window.getJ()));
                addOutBits(charSize());
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==8 - doplneni">
            case 8:
                highlightLine(9);
                for (int i=0;i<window.getJ()+1;i++) {
                    String pom=readNext();
                    if (pom=="") pom=" ";
                    window.addData(pom);
                }
                state=2;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==9- konec">
            case 9:
                write("");
                highlightLine(11);
                repaint();
                return -1;
                // </editor-fold>
        }
        
        repaint();
        return 0;
    }
    
    /**
     * Tato funkce se vola pri inicializaci appletu prohlizecem.
     * Take je volana z metody main pri spusteni jako aplikace.
     * V teto metode se naplni prislusny {@link gui.AlgorithmPanel}
     * lokalizovanou ({@link gui.Localisation }) verzi algoritmu.
     */
    public void init(){
        
        try {
            Localisation.setLocale(new Locale(getParameter("locale")));
        } catch (NullPointerException e) {
            //  Pouziji se defaultni, zde nic nesmi byt, aby to fungovalo i jako aplikace.
        }
        super.init();
        setCodelines(getCodelines());
        setHeadLine(Localisation.getString("77Headline"));
        highlightLine(1);
        
    }
    
    /**
     * Metoda volana pri spousteni jako aplikace.
     * @param args parametr muze byt ve tvaru <PRE>Locale</PRE> pro nastaveni jineho jazyka.
     */
    public static void main(String[] args) {
        Lz77Applet applet;
        if (args.length==1) {
            Localisation.setLocale(new Locale(args[0]));
        }
        applet = new Lz77Applet();
        JFrame window = new JFrame(Localisation.getString("77Title"));
        window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        window.setContentPane(applet);
        applet.init();
        window.pack();
        window.setVisible(true);
    }
    
    private String[] getCodelines(){
        if (lines==null) fillAlgLines();
        return lines;
    }
    
    private void fillAlgLines(){
        lines=new String[algNumLines];
        for (int i=0;i<algNumLines;i++){
            String pom=Localisation.getString("77AlgLine"+(i+1));
            lines[i]=pom;
        }
    }
    
}
