/*
 * Lz78Applet.java
 *
 
 */

package gui;
import gui.parts.Lz78Node;
import gui.utils.FibbCode;
import java.util.Locale;
import javax.swing.JFrame;

/**
 * Trida pro vizualzaci appletu pro kodovani LZ 78.
 * @author Ondrej Hanousek
 */
public class Lz78Applet extends BasicApplet {
    private int state=0;
    private String w="";
    private String X;
    
    private static int algNumLines=14;
    private static String[] lines;
    
    private Lz78Node root=new Lz78Node();
    private Lz78Node actual=root;
    private FibbCode code=new FibbCode(3);
    
    /**
     * Vytvori tridu pro vizualizaci kodovani LZ 78.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala systemove <PRE>Locales</PRE>. To zajistuje
     * zobrazeni popisku a textu v aplikaci ve spravnem jazyce,
     * pokud je k dispozici.
     */
    public Lz78Applet() {
    }
    
    /**
     * Vytvori tridu pro vizualizaci kodovani LZ 78.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala <PRE>Locales</PRE> zadane parametrem. To
     * zajistuje zobrazeni popisku a textu v aplikaci ve spravnem
     * jazyce, pokud je k dispozici.
     * @param l <PRE>Locales </PRE> nastavene pro aplikaci
     */
    public Lz78Applet(Locale l) {
        Localisation.setLocale(l);
    }
    
    
    /**
     * Pretizeni abstraktni metody ze tridy {@link gui.BasicApplet },
     * ktera je volana pri kazdem kroku - tj. po akci tlacitka
     * "krok" nebo pravidelne ve stavu "spusteno".
     * @return 0 - v pripade ze jsou mozne dalsi kroky,
     * -1 - kdyz neni dalsi krok mozny.
     */
    public int doNextStep() {
        switch(state){
            //<editor-fold defaultstate="collapsed" desc=" State==0 - inicializace " >
            case 0:
                highlightLine(1);
                X="";
                state=1;
                actual=root;
                actual.setHighlighted(true);
                addPart(root);
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==1 w="" " >
            case 1:
                highlightLine(2);
                w="";
                state=2;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==2 EOF " >
            case 2:
                highlightLine(3);
                if (!eof()) {
                    state=3;
                    break;
                } else {
                    state=10;
                    break;
                }
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==3 nacteni " >
            case 3:
                X=readNext();
                highlightLine(4);
                state=4;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==4 if  " >
            case 4:
                highlightLine(5);
                if (actual.accepts(X)) {
                    state=5;
                    break;
                } else {
                    state=7;
                    break;
                }
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==5 w=wX  " >
            case 5:
                highlightLine(6);
                actual.highlightedTransition(X);
                actual.setHighlighted(false);
                actual=actual.next(X);
                actual.setHighlighted(true);
                state=6;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==6 endwhile  " >
            case 6:
                write("");
                highlightLine(13);
                state=2;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==7 druha vetev if  " >
            case 7:
                highlightLine(9);
                write("("+actual.getNumber()+","+X+")");
                addOutBits(code.fibbLength(actual.getNumber()));
                addOutBits(charSize());
                state=8;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==8 new node  " >
            case 8:
                highlightLine(10);
                Lz78Node newNode=new Lz78Node();
                actual.addNext(X, newNode);
                addPart(newNode);
                state=9;
                root.updatePosition(0,0);
                
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==9 w=""  " >
            case 9:
                highlightLine(11);
                w="";
                actual.setHighlighted(false);
                actual=root;
                actual.setHighlighted(true);
                state=6;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==10 end  " >
            case 10:
                readNext();
                root.removeAllHighlights();
                highlightLine(14);
                return -1;
                // </editor-fold>
        }
        repaint();
        return 0;
    }
    
    /**
     * Tato funkce se vola pri inicializaci appletu prohlizecem.
     * Take je volana z metody main pri spusteni jako aplikace.
     * V teto metode se naplni prislusny {@link gui.AlgorithmPanel}
     * lokalizovanou ({@link gui.Localisation }) verzi algoritmu.
     */
    public void init(){
        
        try {
            Localisation.setLocale(new Locale(getParameter("locale")));
        } catch (NullPointerException e) {
            //  Pouziji se defaultni, zde nic nesmi byt, aby to fungovalo i jako aplikace.
        }
        super.init();
        setCodelines(getCodelines());
        setHeadLine(Localisation.getString("78Headline"));
        highlightLine(1);
        
    }
    
    /**
     * Metoda volana pri spousteni jako aplikace.
     * @param args parametr muze byt ve tvaru <PRE>Locale</PRE> pro nastaveni jineho jazyka.
     */
    public static void main(String[] args) {
        Lz78Applet applet;
        if (args.length==1) {
            Localisation.setLocale(new Locale(args[0]));
        }
            applet = new Lz78Applet();
        JFrame window = new JFrame(Localisation.getString("78Title"));
        window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        window.setContentPane(applet);
        applet.init();
        window.pack();
        window.setVisible(true);
    }
    
    private String[] getCodelines(){
        if (lines==null) fillAlgLines();
        return lines;
    }
    
    private void fillAlgLines(){
        lines=new String[algNumLines];
        for (int i=0;i<algNumLines;i++){
            String pom=Localisation.getString("78AlgLine"+(i+1));
            lines[i]=pom;
        }
    }
    
    
    /**
     * Zavolano z tridy {@link gui.BasicApplet }, pokud se ma obnovit puvodni
     * nastaveni appletu.
     */
    public void reset(){
        super.reset();
        state=0;
        Lz78Node.reset();
        root=new Lz78Node();
        actual=root;
    }
    
}
