/*
 * SFGui.java
 *
 * Created on 8. jen 2005, 11:53
 *
 */

package gui;
import gui.parts.CharInfo;
import gui.utils.FibbCode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.Locale;
import javax.swing.JFrame;
import gui.utils.CharInfoCountPosComparator;
import gui.utils.Steped;
import java.awt.Color;
import java.util.List;
import gui.utils.ColorCycler;
import gui.parts.SFCharInfoRegion;
import gui.parts.SFCharInfo;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Trida pro vizualizaci Shannon-Fanova kodovani.
 * @author Ondrej Hanousek
 */
public class SFApplet extends BasicApplet {
    
    
    
    private class Split implements Steped{
        private int splitState=0;
        private Split sub;
        private int from=0;             //split - from
        private int to=0;                   // split- to
        private int size=0;             //split size
        private int splitPoint=0;      // split splitpoint
        private int halfSize=0;       // split- velikost pulky S1
        private SFCharInfoRegion s,s1,s2;;
        private String label;
        
        
        public Split(){
        }
        
        public Split(int from, int to, int size, String label)    {
            this.from=from;
            this.to=to;
            this.size=size;
            this.label=label;
            s=new SFCharInfoRegion(colors.getLastColor(),label,true);
            for (int i=0;i<charArray.length;i++){
                s.addInnerPart(charArray[i]);
            }
            addRegion(s);
        }
        
        public Split(int from, int to, int size, SFCharInfoRegion region)    {
            this.from=from;
            this.to=to;
            this.size=size;
            this.s=region;
            
        }
        
        public int doNextStep() {
            if (sub!=null) {
                int ret=sub.doNextStep();
                if (ret==-1) {
                    sub=null;
                    return doNextStep();
                }
                return ret;
            }
            switch (splitState) {
                //<editor-fold defaultstate="collapsed" desc=" State==0 - zobrazeni SPLIT" >
                case 0:
                    s.setBold(true);
                    highlightLine(9);
                    splitState=1;
                    break;
                    // </editor-fold>
                    //<editor-fold defaultstate="collapsed" desc=" State==1 - podminka ve SPLIT" >
                case 1:
                    highlightLine(11);
                    if ((to-from)>1) {
                        splitState=2;
                    } else {
                        splitState=21;
                    }
                    break;
                    // </editor-fold>
                    //<editor-fold defaultstate="collapsed" desc=" State==2 - deleni na Sx1 a Sx2" >
                case 2:
                    highlightLine(13);
                    
                    halfSize=0;
                    int i;
                    int lastCount=0;
                    for (i=from;i<to-1;i++) {
                        lastCount=charArray[i].getCount();
                        halfSize+=lastCount;
                        if (halfSize>=(size/2.0)) {
                            if ((halfSize-size/2.0)<(lastCount/2.0)) {
                                //System.out.println("+");
                                i++;
                            } else {
                                halfSize-=lastCount;
                            }
                            break;
                        }
                    }
                    //    System.out.println(halfSize+" "+ size/2 + " "+lastCount+" "+size);
                    splitPoint=i;
                    
                    s1=new SFCharInfoRegion(colors.getNextColor(), s.getLabel()+"1", false);
                    for (int j=from;j<splitPoint;j++){
                        s1.addInnerPart(charArray[j]);
                    }
                    s2=new SFCharInfoRegion(colors.getLastColor(),s.getLabel()+"2", false);
                    for (int j=splitPoint;j<to;j++){
                        s2.addInnerPart(charArray[j]);
                    }
                    addRegion(s1);
                    addRegion(s2);
                    s.addInnerPart(s1);
                    s.addInnerPart(s2);
                    splitState=3;
                    break;
                    // </editor-fold>
                    //<editor-fold defaultstate="collapsed" desc=" State==3 - pridani 1 k Sx1" >
                case 3:
                    highlightLine(14);
                    for (int j=from;j<splitPoint;j++){
                        charArray[j].addBit("1");
                    }
                    splitState=4;
                    break;
                    // </editor-fold>
                    //<editor-fold defaultstate="collapsed" desc=" State==4 - pridani 0 k Sx2" >
                case 4:
                    highlightLine(15);
                    for (int j=0;j<splitPoint;j++){
                        charArray[j].resetLastBitColor();
                    }
                    for (int j=splitPoint;j<to;j++){
                        charArray[j].addBit("0");
                    }
                    splitState=5;
                    break;
                    // </editor-fold>
                    //<editor-fold defaultstate="collapsed" desc=" State==5 - split Sx1" >
                case 5:
                    for (int j=splitPoint;j<to;j++){
                        charArray[j].resetLastBitColor();
                    }
                    highlightLine(16);
                    s.setBold(false);
                    s1.setBold(true);
                    sub=new Split(from, splitPoint, halfSize,s1);
                    splitState=6;
                    break;
                    // </editor-fold>
                    //<editor-fold defaultstate="collapsed" desc=" State==6 - split Sx2" >
                case 6:
                    s1.setBold(false);
                    s2.setBold(true);
                    highlightLine(17);
                    sub=new Split(splitPoint, to, size-halfSize, s2);
                    splitState=20;
                    break;
                    // </editor-fold>
                    //<editor-fold defaultstate="collapsed" desc=" State==20 - end " >
                case 20:
                    highlightLine(18);
                    s.setBold(true);
                    splitState=21;
                    break;
                    // </editor-fold>
                    //<editor-fold defaultstate="collapsed" desc=" State==21 - |Sx| =1 " >
                case 21:
                    highlightLine(19);
                    s.setBold(true);
                    splitState=22;
                    break;
                    // </editor-fold>
                    //<editor-fold defaultstate="collapsed" desc=" State==22 - end " >
                case 22:
                    s.setBold(false);
                    removeRegion(s);
                    removeRegion(s1);
                    removeRegion(s2);
                    repaint();
                    return -1;
                    // </editor-fold>
                    
            }
            return 0;
        }

        public void steppingEnded() {
        }
    }
    
    private static int algNumLines=19;
    private static String[] lines;
    private ColorCycler colors=new ColorCycler();
    private List<SFCharInfoRegion> closedRegions=new LinkedList<SFCharInfoRegion>();
    
    private int totalchars=0;   //celkem nacteno znaku
    private int state=0;            //stav v algoritmu
    private int position=0;         //pozice "chlivku"
    
    private SFCharInfo[] charArray;
    private Split rootSplit=new Split();
    private SFCharInfoRegion found;
    private boolean selected=false;
    
    private FibbCode len=new FibbCode(8);
    
    private ArrayList<SFCharInfo> characters=new ArrayList<SFCharInfo>();
    /**
     * Vytvori tridu pro vizualizaci Shannon-Fanova kodovani.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala systemove <PRE>Locales</PRE>. To zajistuje
     * zobrazeni popisku a textu v aplikaci ve spravnem jazyce,
     * pokud je k dispozici.
     */
    public SFApplet() {
    }
    
    /**
     * Vytvori tridu pro vizualizaci Shannon-Fanova kodovani.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala <PRE>Locales</PRE> zadane parametrem. To
     * zajistuje zobrazeni popisku a textu v aplikaci ve spravnem
     * jazyce, pokud je k dispozici.
     * @param l <PRE>Locales </PRE> nastavene pro aplikaci
     */
    public SFApplet(Locale l) {
        Localisation.setLocale(l);
    }
    
    private void fillAlgLines(){
        lines=new String[algNumLines];
        for (int i=0;i<algNumLines;i++){
            String pom=Localisation.getString("sfAlgLine"+(i+1));
            lines[i]=pom;
        }
    }
    
    private String[] getCodelines(){
        if (lines==null) fillAlgLines();
        return lines;
    }
    
    
    /**
     * Pretizeni abstraktni metody ze tridy {@link gui.BasicApplet },
     * ktera je volana pri kazdem kroku - tj. po akci tlacitka
     * "krok" nebo pravidelne ve stavu "spusteno".
     * @return 0 - v pripade ze jsou mozne dalsi kroky,
     * -1 - kdyz neni dalsi krok mozny.
     */
    public int doNextStep() {
        switch (state) {
            //<editor-fold defaultstate="collapsed" desc=" State==0 - pocty " >
            case 0:
                highlightLine(2);
                String pom=readNext();
                if (pom.compareTo("")==0) {
                    charArray=characters.toArray(new SFCharInfo[characters.size()]);
                    state=88;
                    break;
                }
                totalchars++;
                SFCharInfo tempInfo=new SFCharInfo(pom);
                int place=characters.indexOf(tempInfo);
                if (place!=-1) {
                    characters.get(place).incrementCounter();
                } else {
                    characters.add(tempInfo);
                    tempInfo.setPosition(position++);
                    addPart(tempInfo);
                }
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==1 - trideni" >
            case 1:
                highlightLine(4);
                Arrays.sort(charArray, new CharInfoCountPosComparator());
                for (int i=0;i<charArray.length;i++){
                    charArray[i].setPosition(i);
                }
                state=2;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==2 - prvni SPLIT" >
            case 2:
                rootSplit=new Split(0, charArray.length, totalchars, "S");
                highlightLine(5);
                state=3;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==3 - SPLIT dokud to jde" >
            case 3:
                
                if (rootSplit.doNextStep()==-1) {
                    rewindInput();
                    highlightLine(6);
                    state=4;
                    for (SFCharInfoRegion r:closedRegions){
                        removePart(r);
                    }
                    closedRegions=new LinkedList<SFCharInfoRegion>();
                    
                }
                
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==4 - vytvoreni vystupu" >
            case 4:
                highlightLine(6);
                pom=readNext();
                if (pom.compareTo("")==0) {
                    state=5;
                    write("");
                    break;
                }
                int ii=0;
                for (ii=0;ii<charArray.length;ii++) {
                    if (charArray[ii].equals(new SFCharInfo(pom))) break;
                }
                removePart(found);
                found=new SFCharInfoRegion(Color.red, java.util.ResourceBundle.getBundle("gui/Locales",getLocale()).getString("found"), false);
                found.addInnerPart(charArray[ii]);
                addPart(found);
                write(charArray[ii].getCode());
                addOutBits2(charArray[ii].getCode().length());
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==5 - konec" >
            case 5:
                removePart(found);
                highlightLine(7);
                repaint();
                return -1;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==88 - vypis cetnosti" >
            case 88:
                highlightLine(3);
                for (char c : VALID.toCharArray()) {
                    SFCharInfo info=new SFCharInfo(c+"");
                    int index=characters.indexOf(info);
                    String out=(index==-1)?len.fibbCode(0):len.fibbCode(characters.get(index).getCount());
                    write(out+" ");
                    addOutBits(out.length());
                }
                write("");
                state=1;
                break;
                // </editor-fold>
                
        }
        repaint();
        return(0);
    }
    private void addRegion(SFCharInfoRegion r){
        addPart(r);
    }
    
    private void removeRegion(SFCharInfoRegion r){
        if (selected) {
            removePart(r);
        }
        if (r!=null)  closedRegions.add(r);
    }
    
    /**
     * Metoda volana pri spousteni jako aplikace.
     * @param args parametr muze byt ve tvaru <CODE>Locale</CODE> pro nastaveni jineho jazyka.
     */
    public static void main(String[] args) {
        SFApplet sfApplet;
      if (args.length==1) {
            Localisation.setLocale(new Locale(args[0]));
        }
            sfApplet = new SFApplet();
       JFrame sfWindow = new JFrame(Localisation.getString("sfTitle"));
        sfWindow.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        sfWindow.setContentPane(sfApplet);
        sfApplet.init();
        sfWindow.pack();
        sfWindow.setVisible(true);
    }
    
    /**
     * Tato funkce se vola pri inicializaci appletu prohlizecem.
     * Take je volana z metody main pri spusteni jako aplikace.
     * V teto metode se naplni prislusny {@link gui.AlgorithmPanel}
     * lokalizovanou ({@link gui.Localisation }) verzi algoritmu.
     */
    public void init(){
        
        try {
            Localisation.setLocale(new Locale(getParameter("locale")));
        } catch (NullPointerException e) {
            //  Pouziji se defaultni, zde nic nesmi byt, aby to fungovalo i jako aplikace.
        }
        CheckBoxPanel p=new CheckBoxPanel(Localisation.getString("sfRecursion"));
        p.addItemStateChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent evt) {
                checkedStateChanged(evt);
            }
        });
        setAdditionalControlsPanel(p);
        super.init();
        setCodelines(getCodelines());
        setHeadLine(Localisation.getString("sfHeadline"));
        highlightLine(1);
        setTwoCounters(true);
    }
    
    /**
     * Zavolano z tridy {@link gui.BasicApplet }, pokud se ma obnovit puvodni
     * nastaveni appletu.
     */
    public void reset() {
        state=0;
        position=0;
        totalchars=0;
        rootSplit=null;
        characters=new ArrayList<SFCharInfo>();
        super.reset();
    }
    
    /**
     * Metoda pro kontrolu, jestli ma retezec dva ruzne znaky. 
     * Nutne, protoze algoritmus neobsahuje vsechny okrajove 
     * podminky.
     * @param s retezec k porovnani
     * @return platnost retezce
     */
    public boolean isValidInput(String s){
        for (int i=0;i<s.length();i++){
            if (s.charAt(i)!=s.charAt(0)) return true;
        }
        return false;
    }
    
    /**
     * Chybova zprava pro neplatny retezec.
     * @return chybova zprava
     */
    public String getInvalidInputMsg(){
        return Localisation.getString("sfInvalidInput");
    }
    
    private void checkedStateChanged(javax.swing.event.ChangeEvent evt) {
        selected=((CheckBoxPanel) evt.getSource()).isSelected();
        if (selected) {
            for (SFCharInfoRegion r:closedRegions){
                //     System.out.println("remove" + r);
                removePart(r);
            }
        } else {
            for (SFCharInfoRegion r:closedRegions){
                addPart(r);
            }
        }
        repaint();
    }
    
}
