/*
 * StatsPanel.java
 *
 * Created on 26. listopad 2005, 15:08
 */

package gui;

import java.text.NumberFormat;

/**
 * Panel se statistickymi informacemi o kompresi. Umoznuje pouziti jednoho nebo dvou pocitadel vystupnich bitu.
 * @author Ondrej Hanousek
 */
public class StatsPanel extends javax.swing.JPanel {
    private int inBits=0;
    private int outBits1=0;
    private int outBits2=0;
    private boolean twoCounters=false;
    
    /**
     *Povoli / zakaze dve pocitadla. Pouziva se tam, kde se uklada statisticky model. 
     */
    public void setTwoCounters(boolean b){
        twoCounters=b;
        plus.setVisible(b);
        plus1.setVisible(b);
        ratioLbl2.setVisible(b);
        outBitsLbl2.setVisible(b);
        updateRatio();
    }
    
    /**
     * Vytvori panel pro statistiky.
     */
    public StatsPanel() {
        initComponents();
        setTwoCounters(false);
    }
    
    /**
     * Prida k 1. pocitadlu vystupnich bitu.
     * @param bits pocet bitu
     */
    public void addOutBits1(int bits){
        outBits1+=bits;
        updateRatio();
    }
    
    /**
     * Prida k 2. pocitadlu vystupnich bitu.
     * @param bits pocet bitu
     */
    public void addOutBits2(int bits){
        outBits2+=bits;
        updateRatio();
    }
    
    /**
     * Nastavi pocet pocitadlu vstupnich bitu.
     * @param bits pocet bitu
     */
    public void setInBits(int bits){
        inBits=bits;
        updateRatio();
    }
    
    /**
     * Vynuluje pocitadlo vystupnich bitu.
     */
    public void resetOutBits() {
        outBits1=0;
        updateRatio();
    }
    
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        inCaption = new javax.swing.JLabel();
        inBitsLbl = new javax.swing.JLabel();
        outCaption = new javax.swing.JLabel();
        outBitsLbl = new javax.swing.JLabel();
        ratioCaption = new javax.swing.JLabel();
        ratioLbl = new javax.swing.JLabel();
        plus = new javax.swing.JLabel();
        outBitsLbl2 = new javax.swing.JLabel();
        plus1 = new javax.swing.JLabel();
        ratioLbl2 = new javax.swing.JLabel();

        setLayout(new java.awt.GridBagLayout());

        inCaption.setText(Localisation.getString("stInput"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 2);
        add(inCaption, gridBagConstraints);
        inCaption.getAccessibleContext().setAccessibleName("");

        inBitsLbl.setText("8 b");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 10);
        add(inBitsLbl, gridBagConstraints);

        outCaption.setText(Localisation.getString("stOutput"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 2);
        add(outCaption, gridBagConstraints);

        outBitsLbl.setText("10 b");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        add(outBitsLbl, gridBagConstraints);

        ratioCaption.setText(Localisation.getString("stRatio"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 2);
        add(ratioCaption, gridBagConstraints);

        ratioLbl.setText("0.11");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        add(ratioLbl, gridBagConstraints);

        plus.setText("+");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        add(plus, gridBagConstraints);

        outBitsLbl2.setText("10 b");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        add(outBitsLbl2, gridBagConstraints);

        plus1.setText("+");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 8;
        add(plus1, gridBagConstraints);

        ratioLbl2.setText("0.11");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 9;
        add(ratioLbl2, gridBagConstraints);

    }
    // </editor-fold>//GEN-END:initComponents
    
    private void updateRatio(){
        inBitsLbl.setText(inBits+" b");
        String inTip=Localisation.getString("stToolTip1")+" "+BasicApplet.charSize()+"b";
        inBitsLbl.setToolTipText(inTip);
        inCaption.setToolTipText(inTip);
        
        if (twoCounters){
            String tooltip4=Localisation.getString("stToolTip4");
            String tooltip5=Localisation.getString("stToolTip5");
            String tooltip6=Localisation.getString("stToolTip6");
            String tooltip7=Localisation.getString("stToolTip7");
            outCaption.setToolTipText(tooltip4 + " + "+tooltip5);
            ratioCaption.setToolTipText(tooltip6+" + "+tooltip7);
            
            outBitsLbl.setText(outBits1+"b");
            outBitsLbl.setToolTipText(tooltip4);
            outBitsLbl2.setText(outBits2+"b");
            outBitsLbl2.setToolTipText(tooltip5);
            
            float f1=(float) outBits1/(float) inBits;
            float f2=(float) outBits2/(float) inBits;
            NumberFormat nf=NumberFormat.getInstance(Localisation.getLocale());
             
            ratioLbl.setText(nf.format(f1));
            ratioLbl.setToolTipText(tooltip6);
            ratioLbl2.setText(nf.format(f2));
            ratioLbl2.setToolTipText(tooltip7);
            
        } else {
            outBitsLbl.setText(outBits1+" b");
            String tooltip=Localisation.getString("stToolTip2")+" "+BasicApplet.charSize()+"b";
            outBitsLbl.setToolTipText(tooltip);
            outCaption.setToolTipText(tooltip);
            float f=(float) outBits1/(float) inBits;
            NumberFormat nf=NumberFormat.getInstance(Localisation.getLocale());
            String ratioToolTip=Localisation.getString("stToolTip3");
            ratioLbl.setText(nf.format(f));
            ratioLbl.setToolTipText(ratioToolTip);
            ratioCaption.setToolTipText(ratioToolTip);
        }
        
        
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel inBitsLbl;
    private javax.swing.JLabel inCaption;
    private javax.swing.JLabel outBitsLbl;
    private javax.swing.JLabel outBitsLbl2;
    private javax.swing.JLabel outCaption;
    private javax.swing.JLabel plus;
    private javax.swing.JLabel plus1;
    private javax.swing.JLabel ratioCaption;
    private javax.swing.JLabel ratioLbl;
    private javax.swing.JLabel ratioLbl2;
    // End of variables declaration//GEN-END:variables
    
}
