/*
 * ARCharInfo.java
 *
 * Created on 26. listopad 2005, 16:23
 */

package gui.parts;
import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;

/**
 * Trida pro informace o znaku v artmetickem
 * kodovani.
 * @author Ondrej Hanousek
 */
public class ARCharInfo extends CharInfo {
   /**
    * sirka obdelnicku kolem znaku
    */
   public static final int BOX_WIDTH=40;
    /**
     * vyska obdelnicku kolem znaku
     */
    public static final int BOX_HEIGHT=25;
    /**
     * svisle mezery mezi obdelnicky.
     */
    public static final int Y_SPACING=20;
    /**
     * Posunuti obdelnicku od leveho okraje
     */
    public static final int X_OFFSET=10;
    /**
     * posunuti obdelnicku od horniho okraje
     */
    public static final int Y_OFFSET=12;

    private double from=0;
    private double to=0;
    
    private int position=0;
    
    /**
     * Vytvori objekt se zadanym znakem.
     * @param character znak
     */
    public ARCharInfo(String  character){
        setCharacter(character);
    }

    /**
     * Vraci vertikalni pozici objektu.
     * @return vertikalni poradi objektu
     */
    public int getPosition() {
        return position;
    }

    /**
     * Vykresli graficky objekt.
     * @param g objekt, na ktery se kresli
     */
    public void paintPart(java.awt.Graphics g) {
        Graphics2D g2d=(Graphics2D) g;
        Font def=g2d.getFont();
        g2d.setStroke(new BasicStroke(2));
        g2d.drawRoundRect(x, y, BOX_WIDTH, BOX_HEIGHT, 4, 4);
        g2d.setFont(def.deriveFont(Font.BOLD,16f));
        
        FontMetrics m=g2d.getFontMetrics();
        String label=getCharacter()+" - "+getCount();
        int textX=x+5;
        int textY=(int) (y+Math.round((double) BOX_HEIGHT/2.0+(double) m.getAscent()/2.0)-3);
        g2d.drawChars(label.toCharArray(), 0, label.length(), textX, textY);
        g2d.setFont(def);
    }

    /**
     * Minimalni opisujici obdelnik.
     * @return minimalni obdelnik
     */
    public java.awt.Rectangle getBounds() {
      return new Rectangle(x-4, y-Y_OFFSET, BOX_WIDTH+8, BOX_HEIGHT+Y_OFFSET+4);
    }
 
  /**
   * Nastavi svislou pozici objektu s informaci o znaku v 
   * aritmeticke kompresi
   * @param position svisla pozice
   */
  public void setPosition(int position){
        x=4;
        y=position*(Y_SPACING+BOX_HEIGHT)+Y_OFFSET;
     //   System.out.println("p"+position);
        this.position=position;
    }
  
  
  /**
   * nastavi spodni mez prideleneho intervalu
   * @param f spodni mez
   */
  public void setFrom(double f){
      from=f;
  }
  
  /**
   * nastavi horni mez prideleneho intervalu.
   * @param f horni mez
   */
  public void setTo(double f){
      this.to=f;
  }
  
  /**
   * vraci dolni mez prislusneho podintervalu.
   * @return dolni mez
   */
  public double getFrom(){
      return from;
  }
  /**
   * horni mez prislusneho podintervalu.
   * @return horni mez
   */
  public double getTo(){
      return to;
  }
}
