/*
 * ARInterval.java
 *
 * Created on 27. listopad 2005, 11:00
 */

package gui.parts;
import gui.Localisation;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.GeneralPath;
import java.text.NumberFormat;
import java.util.Collection;



/**
 * Trida pro zobrazeni intervalu v aritmeticke kompresi.
 * @author Ondrej Hanousek
 */
public class ARInterval extends Part {
    /**
     * Sirka obdelniku
     */
    public static final int BOX_WIDTH=10;
    /**
     * Sirka obdelniku
     */
    public static final int BOX_HEIGHT=250;
    /**
     * sirka mezi dvema objekty
     */
    public static final int X_SPACING=100;
    /**
     * odsazeni shora
     */
    public static final int Y_OFFSET=12;
    /**
     * odsazeni od leveho okraje
     */
    public static final int X_OFFSET=10;
    
    private double from;
    private double to;
    private Collection<ARCharInfo> units;
    private int sum=0;
    private int widthPlus=40;
    private ARInterval next=null;
    private int addedSum=0;
    private boolean highlighted=true;
    private boolean painted=false;
    
    /**
     * Vytvori interval s danou horni a dolni hranici.
     * @param from dolni mez
     * @param to horni mez
     */
    public ARInterval(double from, double to) {
        this.from=from;
        this.to=to;
    }
    
    /**
     * Vykresleni objektu.
     * @param g objekt, na ktery se kresli
     */
    public void paintPart(java.awt.Graphics g) {
        Graphics2D g2d=(Graphics2D) g;
        addedSum=0;
        g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));
        if (highlighted) g2d.setColor(Color.BLUE);
        g2d.drawLine(x+BOX_WIDTH/2, y, x+BOX_WIDTH/2, y+BOX_HEIGHT);
        g2d.drawLine(x, y, x+BOX_WIDTH, y);
        g2d.drawLine(x, y+BOX_HEIGHT, x+BOX_WIDTH, y+BOX_HEIGHT);
        g2d.setColor(Color.BLACK);
      
        NumberFormat nf=NumberFormat.getInstance(Localisation.getLocale());
        nf.setMaximumFractionDigits(15);
        nf.setMinimumFractionDigits(1);
        String fromS=nf.format(from);
        String toS=nf.format(to);
        FontMetrics m=g2d.getFontMetrics();
        g2d.drawChars(fromS.toCharArray(), 0, fromS.length(), x+BOX_WIDTH+2, y+BOX_HEIGHT+m.getAscent());
        g2d.drawChars(toS.toCharArray(), 0, toS.length(), x+BOX_WIDTH+2, y);
        widthPlus=(int) Math.max(widthPlus,m.getStringBounds(fromS,g2d).getWidth());
        widthPlus=(int) Math.max(widthPlus,m.getStringBounds(toS,g2d).getWidth());
        
        double intervalTo=0;
        int linePosition=y+BOX_HEIGHT;
        
        //Kresleni intervalu
        if (units!=null) {
            for (ARCharInfo i:units) {
                int  prePosition=linePosition;
                i.setFrom(intervalTo);
                addedSum+=i.getCount();
                intervalTo=((double) addedSum/ (double) sum) * (to-from);
                i.setTo(intervalTo);
                linePosition=(int) Math.round(y+BOX_HEIGHT-(BOX_HEIGHT)*intervalTo/(to-from));
                if (addedSum<sum) {
                    if (highlighted) g2d.setColor(Color.BLUE);
                    g2d.drawLine(x,linePosition , x+BOX_WIDTH,linePosition);
                    String lineLabel=nf.format(intervalTo);
                    g2d.setColor(Color.BLACK);
                    g2d.drawChars(lineLabel.toCharArray(), 0, lineLabel.length(), x+BOX_WIDTH+2, linePosition-3);
                    widthPlus=(int) Math.max(widthPlus,m.getStringBounds(lineLabel,g2d).getWidth());
                }
                String intervalCaption=i.getCharacter();
                m=g2d.getFontMetrics();
                int captionWidth=m.charsWidth(intervalCaption.toCharArray(), 0, intervalCaption.length());
                int captionPos=prePosition+(linePosition-prePosition)/2+m.getAscent()/2;
                g2d.drawChars(intervalCaption.toCharArray(), 0, intervalCaption.length(),x-captionWidth/2-5,captionPos );
                
            }
            
            //Kresleni spojnic
            if (next!=null) {
                g2d.setStroke(new BasicStroke(1, BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));
                int fromPos=(int) Math.round(y+BOX_HEIGHT-(BOX_HEIGHT)*next.getFrom()/(to-from));
                int toPos=(int) Math.round(y+BOX_HEIGHT-(BOX_HEIGHT)*next.getTo()/(to-from));
                
                int halfX=x+BOX_WIDTH+widthPlus+X_SPACING/2;
                
                GeneralPath fromPath=new GeneralPath();
                fromPath.moveTo(x+BOX_WIDTH, fromPos);
                fromPath.lineTo(x+BOX_WIDTH+widthPlus,fromPos);
                
                fromPath.curveTo(halfX, fromPos, halfX, next.getY()+BOX_HEIGHT, next.getX(),next.getY()+BOX_HEIGHT);
                
                GeneralPath toPath=new GeneralPath();
                toPath.moveTo(x+BOX_WIDTH, toPos);
                toPath.lineTo(x+BOX_WIDTH+widthPlus,toPos);
                toPath.curveTo(halfX, toPos, halfX, next.getY(), next.getX(),next.getY());
                
                
                g2d.draw(fromPath);
                g2d.draw(toPath);
            }
            
        }
        if (!painted) {
            fireSizeChangeEvent(new SizeChange(this));
            painted=true;
        }
    }
    
    /**
     * odsazeni zleva
     */
    public void setUnits(Collection<ARCharInfo> c){
        units=c;
        sum=0;
        for (ARCharInfo i:units) {
            sum+=i.getCount();
        }
     //   System.out.println(sum);
        fireSizeChangeEvent(new SizeChange(this));
    }
    
    public java.awt.Rectangle getBounds() {
        return new Rectangle(x-X_OFFSET,y-Y_OFFSET,BOX_WIDTH+widthPlus+X_OFFSET+5, BOX_HEIGHT+2*Y_OFFSET+15);
    }
    
    public double getFrom(){
        return from;
    }
    
    public double getTo(){
        return to;
    }
    
    public void setNext(ARInterval next){
        this.next=next;
    }
    
    public void setHighlighted(boolean b){
        this.highlighted=b;
    }
}
