/*
 * FGKCharInfo.java
 *
 * Created on 6. listopad 2005, 22:03
 *
 */

package gui.parts;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;

/**
 * Graficky objekt v adaptivnim huffmanovi.
 * @author Ondrej Hanousek
 */
public class FGKCharInfo extends CharInfo implements Comparable {
    /**
     * Sirka obdelniku
     */
    public static final int BOX_WIDTH=50;
    /**
     * Sirka obdelniku
     */
    public static final int BOX_HEIGHT=40;
    /**
     * vyska vodorovne mezery
     */
    public static final int Y_SPACING=70;
    /**
     * sirka mezi dvema objekty
     */
    public static final int X_SPACING=50;
    /**
     * sirka mezi dvema objekty
     */
    public static final int X_OFFSET=10;
    /**
     * odsazeni shora
     */
    public static final int Y_OFFSET=10;
    /**
     * barva parametru poradi
     */
    public static final Color ORDER_COLOR=Color.BLUE;
    /**
     * barva parametru poradi
     */
    public static final Color CODE_COLOR=Color.ORANGE;
    /**
     * barva pocitadla
     */
    public static final Color COUNT_COLOR=Color.GRAY;
    /**
     * parva pocitadla
     */
    public static final Color SELECTED_COLOR=Color.GREEN;
    
    
    private FGKCharInfo one,zero,parent;
    private boolean zeroNode=false;
    private int order;
    private boolean highlighted=false;
    private static int nodesCount=0;
    
    /**
     * Graficky objekt v adaptivnim huffmanovi.
     * @param s znak
     */
    public FGKCharInfo(String s) {
        nodesCount++;
        setCharacter(s);
    }
    
    public FGKCharInfo(){
        nodesCount++;
        zeroNode=true;
        setCount(0);
    }
    
    public FGKCharInfo(FGKCharInfo zero, FGKCharInfo one){
        nodesCount++;
        this.zero=zero;
        this.one=one;
        one.setParent(this);
        zero.setParent(this);
        one.moveTo(x-X_OFFSET-X_SPACING, y-Y_OFFSET+Y_SPACING);
        zero.moveTo(x-X_OFFSET+X_SPACING,y-Y_OFFSET+Y_SPACING);
        setCount(1);
        //  System.out.println(zero);
    }
    
    public void finalize(){
        nodesCount--;
    }
    
    public void setZero(FGKCharInfo node){
        zeroNode=false;
        this.zero=node;
        zero.moveTo(x-X_SPACING,y+Y_SPACING);
        zero.setParent(this);
    }
    
    public void setOne(FGKCharInfo node){
        zeroNode=false;
        this.one=node;
        one.moveTo(x+X_SPACING, y+Y_SPACING);
        one.setParent(this);
    }
    
    
    public int getLevel(){
        if (!isRoot()) return parent.getLevel()+1;
        return 0;
    }
    
    public void setParent(FGKCharInfo parent){
        this.parent=parent;
    }
    
    public void updateCount(){
        int p=0;
        if (!isLeaf()) {
            zero.updateCount();
            one.updateCount();
            setCount(zero.getCount()+one.getCount());
        }
        
    }
    
    public void updateChildrenCodes(){
        if (!isLeaf()) {
            zero.setCode(getCode()+"0");
            one.setCode(getCode()+"1");
            zero.updateChildrenCodes();
            one.updateChildrenCodes();
        }
    }
    
    public FGKCharInfo getParent(){
        return parent;
    }
    
    
    public boolean isLeaf(){
        return (zero==null);
    }
    
    public boolean isZeroNode(){
        return zeroNode;
    }
    
    
    /**
     * Vykresleni objektu.
     * @param g1 objekt, na ktery se kresli
     */
    public void paintPart(Graphics g1) {
        
        Graphics2D g=(Graphics2D) g1;
        g.setColor(Color.BLACK);
        Font def=g.getFont();
        g.setStroke(new BasicStroke(2));
        String popis;
        if (isLeaf()){ //ZeroNode
            if (isZeroNode()) {
                popis="ZERO";
                g.drawRoundRect(x,y,BOX_WIDTH, BOX_HEIGHT, 3,3);
            } else { //list se znakem
                popis=getCharacter() + " - "+ getCount();
                g.drawRect(x,y,BOX_WIDTH,BOX_HEIGHT);
            }
            
            //vypis "ZERO" nebo znaku a poctu
            g.setColor(COUNT_COLOR);
            g.setFont(def.deriveFont(Font.BOLD,13f));
            int w=g.getFontMetrics().charsWidth(popis.toCharArray(), 0, popis.length());
            g.drawChars(popis.toCharArray(), 0, popis.length(), x+BOX_WIDTH/2-w/2, y+18);
            //code
            g.setColor(CODE_COLOR);
            g.setFont(def.deriveFont(Font.BOLD,10f));
            g.drawChars(getCode().toCharArray(), 0, getCode().length(), x+4, y+32);
            
            //order
            g.setColor(ORDER_COLOR);
            g.setFont(def.deriveFont(Font.BOLD,10f));
            popis=order+"";
            w=g.getFontMetrics().charsWidth(popis.toCharArray(), 0, popis.length());
            g.drawChars(popis.toCharArray(), 0, popis.length(), x+BOX_WIDTH-w-2, y+32);
            g.setColor(Color.BLACK);
        } else {    //Kresleni uzlu a spojnic
            
            //pocet
            g.drawOval(x, y, BOX_WIDTH,BOX_HEIGHT);
             g.setColor(COUNT_COLOR);
            popis=getCount()+"";
            g.setFont(def.deriveFont(Font.BOLD,13f));
            int w=g.getFontMetrics().charsWidth(popis.toCharArray(), 0, popis.length());
            g.drawChars(popis.toCharArray(), 0, popis.length(), x+BOX_WIDTH/2-w/2, y+18);
            
            //order
            g.setColor(ORDER_COLOR);
            g.setFont(def.deriveFont(Font.BOLD,10f));
            popis=order+"";
            w=g.getFontMetrics().charsWidth(popis.toCharArray(), 0, popis.length());
            g.drawChars(popis.toCharArray(), 0, popis.length(), x+BOX_WIDTH/2-w/2, y+34);
            
            //spojnice
            g.setColor(Color.BLACK);
            g.setStroke(new BasicStroke(1));
            g.setFont(g.getFont().deriveFont(Font.PLAIN,10f));
            drawConnectingCurve(g,"1",x+(BOX_WIDTH/2), y+BOX_HEIGHT,
                    one.getX()+BOX_WIDTH/2,one.getY());
            g.setFont(g.getFont().deriveFont(Font.PLAIN,10f));
            drawConnectingCurve(g,"0",x+(BOX_WIDTH/2),y+BOX_HEIGHT,
                    zero.getX()+BOX_WIDTH/2,zero.getY());
        }
        if (highlighted) {
            g.setColor(SELECTED_COLOR);
            g.drawRect(x-2,y-2,BOX_WIDTH+4,BOX_HEIGHT+4);
            g.setColor(Color.BLACK);
        }
        //    g.draw(getBounds());
    }
    
    
    public int getPosition() {
        return 0;
    }
    
    /**
     * Minimalni opisujici obdelnik.
     * @return minimalni obdelnik
     */
    public java.awt.Rectangle getBounds() {
        return new Rectangle(x-4, y-4, BOX_WIDTH+8, BOX_HEIGHT+8);
    }
    
    public boolean isRoot(){
        return parent==null;
    }
    
    public int getOrder(){
        return order;
    }
    
    public void setOrder(int order){
        this.order=order;
    }
    
    
    public int compareTo(Object o) {
        FGKCharInfo i=(FGKCharInfo) o;
        return this.order-i.order;
    }
    
    public String toString(){
        if (isLeaf()) return getOrder()+" "+getCharacter()+" "+getCount();
        else
            return getOrder()+": "+zero.getOrder()+", "+one.getOrder();
    }
    
    public void changeChildren(FGKCharInfo currentNode, FGKCharInfo newNode){
        
        if (currentNode==zero) {
            zero=newNode;
        } else if (currentNode==one){
            one=newNode;
        } else System.err.println("CHYBA!");
        //     updatePositions();
        
    }
    
    public void setHighlighted(boolean b){
        this.highlighted=b;
    }
    
    public void updatePositions(){
        if (!isLeaf()) {
            int mult=1;
            if (!zero.isLeaf() && !one.isLeaf())  {
                mult=2;
                if (!zero.getOne().isLeaf() || !one.getZero().isLeaf()) {
                    mult=3;
                    if (!zero.getOne().isLeaf() && !one.getZero().isLeaf()) {
                        mult=4;
                    }
                }
            }
            zero.moveTo(x- (X_SPACING*mult),y+Y_SPACING);
            one.moveTo(x+ (X_SPACING*mult), y+Y_SPACING);
            zero.updatePositions();
            one.updatePositions();
        }
        fireSizeChangeEvent(new SizeChange(this));
    }
    
    public FGKCharInfo getZero(){
        return zero;
    }
    
    public FGKCharInfo getOne(){
        return one;
    }
}
