/*
 * Lz77Window.java
 *
 * Created on 4. prosinec 2005, 13:10
 *
 */

package gui.parts;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.GeneralPath;

/**
 * vytvori uzel ad. huffmana s danym znakem.
 * @author Ondrej Hanousek
 */
public class Lz77Window extends Part {
    
    public static int SMALL_RECT_SIZE=30;
    /**
     * odsazeni
     */
    public static int X_OFFSET=15;
    /**
     * odsazeni shora
     */
    public static int Y_OFFSET=30;
    
    private int viewSize;
    private int codedSize;
    private String data="";
    
    private boolean showI=false;
    private boolean showJ=false;
    private boolean showX=false;
    private boolean showPrefix=false;
    
    int foundStart=0;
    int foundLength=0;
    
    /**
     * Okno v kodovani LZ 77.
     * @param codedSize velikost zakodovane casti
     * @param viewSize velikost vyhledu
     */
    public Lz77Window(int codedSize, int viewSize) {
        this.viewSize=viewSize;
        this.codedSize=codedSize;
        for (int i=0;i<codedSize+viewSize;i++) {
            data=data + " ";
        }
    }
    
    private static GeneralPath getHorizontalBracketUp(int x,int width,int y){
        int centerX=x+width/2;
        GeneralPath p=new GeneralPath();
        p.moveTo(x,y);
        p.curveTo(x,y-10,centerX,y,centerX,y-10);
        p.curveTo(centerX,y,x+width,y-10,x+width,y);
        return p;
    }
    
    private static GeneralPath getHorizontalBracketDown(int x,int width,int y){
        int centerX=x+width/2;
        GeneralPath p=new GeneralPath();
        p.moveTo(x,y);
        p.curveTo(x,y+10,centerX,y,centerX,y+10);
        p.curveTo(centerX,y,x+width,y+10,x+width,y);
        return p;
    }
    
    
    /**
     * Vykresleni objektu.
     * @param g objekt, na ktery se kresli
     */
    public void paintPart(java.awt.Graphics g) {
        Graphics2D g2d=(Graphics2D) g;
        g2d.setColor(Color.BLACK);
        
        
        if (showX) {
            String pom="X = "+data.charAt(codedSize+foundLength);
            g2d.setStroke(new BasicStroke(4,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));
            g2d.setColor(Color.YELLOW);
            g2d.drawRect(X_OFFSET+(codedSize+foundLength)*SMALL_RECT_SIZE+5,Y_OFFSET,SMALL_RECT_SIZE,SMALL_RECT_SIZE);
            g2d.setColor(Color.BLACK);
            g2d.setStroke(new BasicStroke(1,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));
            g2d.draw(getHorizontalBracketUp(X_OFFSET+(codedSize+foundLength)*SMALL_RECT_SIZE+5,SMALL_RECT_SIZE,Y_OFFSET));
            int sub=(g2d.getFontMetrics().charsWidth(pom.toCharArray(),0,pom.length()))/2;
            g2d.drawChars(pom.toCharArray(),0,pom.length(),X_OFFSET+(codedSize+foundLength)*SMALL_RECT_SIZE+5+SMALL_RECT_SIZE/2-sub,Y_OFFSET-10);
        }
        
        int from=0;
        int length=0;
        
        if (showI) {
            String pom="i = "+foundStart;
            if (foundStart!=0) {
                
                from=(codedSize-foundStart)*SMALL_RECT_SIZE;
                length=(foundLength*SMALL_RECT_SIZE);
                if ((from+length)>codedSize*SMALL_RECT_SIZE) length+=5;
                
                g2d.setStroke(new BasicStroke(1,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));
                g2d.draw(getHorizontalBracketDown(X_OFFSET+from,foundStart*SMALL_RECT_SIZE,Y_OFFSET+SMALL_RECT_SIZE));
                
                int sub=(g2d.getFontMetrics().charsWidth(pom.toCharArray(),0,pom.length()))/2;
                g2d.drawChars(pom.toCharArray(),0,pom.length(),X_OFFSET+from+(foundStart*SMALL_RECT_SIZE/2)-sub,Y_OFFSET+SMALL_RECT_SIZE+10+g2d.getFontMetrics().getHeight());
                
                g2d.setStroke(new BasicStroke(4,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));
                g2d.setColor(Color.GREEN);
                g2d.drawRect(X_OFFSET+(codedSize*SMALL_RECT_SIZE)+5,Y_OFFSET,(foundLength*SMALL_RECT_SIZE),SMALL_RECT_SIZE);
                
                g2d.setColor(Color.BLUE);
                g2d.drawRect(X_OFFSET+from,Y_OFFSET,length,SMALL_RECT_SIZE);
                
                g2d.setColor(Color.BLACK);
                
            } else {
                g2d.drawChars(pom.toCharArray(), 0, pom.length(), X_OFFSET + 20, Y_OFFSET+SMALL_RECT_SIZE+15);
            }
        }
        
        if (showJ) {
            String pom="j = "+foundLength;
            if (foundLength!=0) {
                g2d.setStroke(new BasicStroke(1,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));
                g2d.draw(getHorizontalBracketUp(X_OFFSET+from,length,Y_OFFSET));
                int sub=(g2d.getFontMetrics().charsWidth(pom.toCharArray(),0,pom.length()))/2;
                g2d.drawChars(pom.toCharArray(),0,pom.length(),X_OFFSET+from+(length)/2-sub,Y_OFFSET-10);
            } else {
                g2d.drawChars(pom.toCharArray(), 0, pom.length(), X_OFFSET + 60, Y_OFFSET+SMALL_RECT_SIZE+15);
            }
        }
        
        
        
        
        //kresleni chlivku s daty
        g2d.setStroke(new BasicStroke(2,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));
        g2d.drawRoundRect(X_OFFSET,Y_OFFSET,(codedSize+viewSize)*SMALL_RECT_SIZE+5,SMALL_RECT_SIZE , 2,2);
        
        char[] dataArray=data.toCharArray();
        int i=1;
        
        for (i=1;i<=codedSize+viewSize;i++) {
            int x=i*SMALL_RECT_SIZE+X_OFFSET;
            if (i>codedSize) x+=5;
            if (i<codedSize+viewSize) g2d.drawLine(x, Y_OFFSET, x, Y_OFFSET+SMALL_RECT_SIZE);
            if ((dataArray.length-(codedSize+viewSize)+i)>0) {
                char[] c=new char[1];
                c[0]=dataArray[(dataArray.length-(codedSize+viewSize)+i)-1];
                int cw=g2d.getFontMetrics().charWidth(c[0]);
                g2d.drawChars(c, 0,1, x-(SMALL_RECT_SIZE+cw)/2, Y_OFFSET+SMALL_RECT_SIZE-10);
            }
        }
        int x=codedSize*SMALL_RECT_SIZE+X_OFFSET+5;
        g2d.drawLine(x, Y_OFFSET, x, Y_OFFSET+SMALL_RECT_SIZE);
        
        
        
        
    }
    
    public boolean isViewEmpty(){
        return (data.substring(codedSize).trim().length()==0);
    }
    
    public void findPrefix(){
        foundStart=0;
        foundLength=0;
        
        for (int i=viewSize-1;i>=1;i--) {
            String aData=data.substring(0,codedSize+i-1);
            int tmpFound=aData.lastIndexOf(data.substring(codedSize,codedSize+i));
            if (tmpFound>-1) {
                foundStart=codedSize-tmpFound;
                foundLength=i;
                break;
            }
        }
        if (foundLength!=0) {
            if (data.charAt(codedSize+foundLength)==' ') {
                foundLength--;
                if (foundLength==0) foundStart=0;
            }
        }
        
        //    System.out.println("i: "+foundLength+" j: "+foundStart);
        showPrefix=true;
    }
    
    public void addData(String s){
        data=data+s;
        if (data.length()>viewSize+codedSize) data= data.substring(data.length()-(viewSize+codedSize));
    }
    
    public void showI(boolean b){
        showI=b;
    }
    
    public void showJ(boolean b){
        showJ=b;
    }
    
    public void showX(boolean b){
        showX=b;
    }
    
    public void showPrefix(boolean b){
        showPrefix=b;
    }
    
    public String getOutput() {
        return "("+foundStart+", "+ foundLength+", "+data.charAt(codedSize+foundLength)+")";
    }
    
    public int getI(){
        return foundStart;
    }
    
     public int getJ(){
        return foundLength;
    }
    
    public java.awt.Rectangle getBounds() {
        return new Rectangle(0,0,X_OFFSET+SMALL_RECT_SIZE*(codedSize+viewSize)+20,Y_OFFSET+SMALL_RECT_SIZE+40);
    }
    
}
