/*
 * Lz78Node.java
 *
 * Created on 18. leden 2006, 22:31
 *
 */

package gui.parts;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.Arrays;
import java.util.HashMap;

/**
 * Uzel ve slovniku pri kodovani LZ 78
 * @author Ondrej Hanousek
 */
public class Lz78Node extends Part {
    
    /**
     * vyska mezi vodorovnymi radami ve strome
     */
    public static final int LINE_HEIGHT=90;
    /**
     * sirka pro jeden uzel
     */
    public static final int WIDTH=60;
    private static int counter=0;
    
    private int number=0;
    
    private HashMap<String, Lz78Node> succesors=new HashMap<String,Lz78Node>();
    
    private int width=WIDTH;
    private boolean highlighted=false;
    private String highlightedTransition="";
    
    /**
     * sirka pro jeden uzel
     */
    public Lz78Node() {
        number=counter++;
    }
    
    public void addNext(String character, Lz78Node node) {
        succesors.put(character, node);
    }
    
    public int getNumber(){
        return number;
    }
    
    /**
     * Minimalni opisujici obdelnik.
     * @return minimalni obdelnik
     */
    public Rectangle getBounds() {
        return new Rectangle(x,y,width,WIDTH);
    }
    
    public int getXCenter() {
        return x+width/2+10-WIDTH/2+(WIDTH-20)/2;
    }
    
    
    /**
     * Vykresleni objektu.
     * @param g1 objekt, na ktery se kresli
     */
    public void paintPart(java.awt.Graphics g1) {
        //ovalek
        
        Graphics2D g=(Graphics2D) g1;
        //      g.draw(getBounds());
        
        g.setStroke(new BasicStroke(2.0f));
        g.drawOval(x+width/2+10-WIDTH/2, y+10, WIDTH-20, WIDTH-20);
        if (highlighted) {
            g.setColor(Color.RED);
            g.drawOval(x+width/2+8-WIDTH/2, y+8, WIDTH-16, WIDTH-16);
            g.setColor(Color.BLACK);
        }
        
        
        //spojnice
        g.setFont(g.getFont().deriveFont(Font.PLAIN,14f));
        for (String label: succesors.keySet()) {
            if (label==highlightedTransition) {
                g.setColor(Color.RED);
            } else {
                g.setColor(Color.BLACK);
            }
            Lz78Node node=succesors.get(label);
            Part.drawConnectingCurve(g, label, getXCenter(), y+WIDTH-10, node.getXCenter(), node.getY()+10);
        }
        
        //popisek stavu
        String caption=Integer.toString(number);
        g.setFont(g.getFont().deriveFont(Font.BOLD,22));
        int caption_x=getXCenter()-(g.getFontMetrics().charsWidth(caption.toCharArray(),0, caption.length()))/2;
        g.drawChars(caption.toCharArray(),0,caption.length(),caption_x, y+WIDTH/2+8) ;
        
        
        
    }
    
    public void updatePosition(int x, int y) {
        this.y=y;
        this.x=x;
        String[] keys=new String[succesors.keySet().size()];
        succesors.keySet().toArray(keys);
        Arrays.sort(keys);
     //   System.out.println(Arrays.toString(keys));
        int actual=x;
        for (String character : keys) {
            succesors.get(character).updatePosition(actual, y+LINE_HEIGHT);
            actual+=succesors.get(character).getWidth();
        }
        width=getWidth();
    }
    
    public void setHighlighted(boolean b){
        this.highlighted=b;
    }
    
    public int getWidth(){
        if (succesors.isEmpty()) return WIDTH;
        else {
            int ret=0;
            for (Lz78Node p : succesors.values()) {
                ret+=p.getWidth();
            }
            return ret;
        }
    }
    
    public boolean accepts(String character) {
        return succesors.containsKey(character);
    }
    
    public Lz78Node next(String character){
        return succesors.get(character);
    }
    
    public void highlightedTransition(String character){
        highlightedTransition=character;
    }
    
    public void removeAllHighlights(){
        highlighted=false;
        highlightedTransition="";
    }
    
    
    public static void reset(){
        counter=0;
    }
    
    
    
}
