/*
 * Part.java
 *
 * Created on 4. jen 2005, 14:23
 *
 */

package gui.parts;

import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.GeneralPath;
import javax.swing.event.EventListenerList;

/**
 * Superclass for graphical parts.
 */
public abstract class Part {
        /**
         * odsazeni zleva
         */
        public static final int X_OFFSET=15;
        /**
         * odsazeni shora
         */
        public static final int Y_OFFSET=15;
    
        protected int x=X_OFFSET,y=Y_OFFSET;
    
       public static void drawConnectingCurve(Graphics2D g, String s,int x1,int y1, int x2, int y2){
        GeneralPath gp=new GeneralPath();
        gp.moveTo(x1,y1);
        // gp.curveTo(x1-10,y1+20, x2+10, y1+20, x2,y2);
        int px=Math.min(x1,x2)+Math.abs(x1-x2)/2;
        int py=Math.min(y1,y2)+Math.abs(y1-y2)/2;
        int diff=x1<x2?10:-10;
        gp.quadTo(px+diff,py-Math.abs(diff),x2, y2);
        // gp.lineTo(px+diff,py-Math.abs(diff));
        //  gp.lineTo(x2,y2);
        // gp.closePath();
        g.setStroke(new BasicStroke(2,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));
        g.draw(gp);
      //  g.setFont(g.getFont().deriveFont(12f));
        int w=g.getFontMetrics().charsWidth(s.toCharArray(), 0, s.length());
        diff=x1<x2?18-w:-18;
        g.drawChars(s.toCharArray(), 0, s.length(), px+diff, py-4);
    }
    
    
    private EventListenerList listenerList=new EventListenerList();
    
    /**
     * Method for registering listener for Part bounds change.
     * @param listener Listener for event.
     */
    public void addSizeChangeEventListener(SizeChangeListener listener) {
        listenerList.add(SizeChangeListener.class, listener);
    }
    

    /**
     * This method allows removing listener for Part bounds change.
     * @param listener Listener for event.
     */
    public void removeSizeChangeEventListener(SizeChangeListener listener) {
        listenerList.remove(SizeChangeListener.class, listener);
    }
    
    protected  void fireSizeChangeEvent(SizeChange evt){
        Object[] listeners = listenerList.getListenerList();
        for (int i=0; i<listeners.length; i+=2) {
            if (listeners[i]==SizeChangeListener.class) {
                ((SizeChangeListener)listeners[i+1]).sizeChanged(evt);
            }
        }
    }
    
    /**
     * This method is called when part is painted.
     * @param g objekt, na ktery se kresli
     */
    public abstract void paintPart(Graphics g);
    /**
     * This method returns ... rectangle.
     * @return Rectangle bounds.
     */
    public abstract Rectangle getBounds();
    /**
     * Move Part x pixels right and y pixels left.
     * @param x x-offset from current position.
     * @param y y-offset from current position
     */
   public void move(int x, int y) {
        this.x-=x;
        this.y-=y;
    }

    public void moveTo(int x, int y) {
        this.x=x;
        this.y=y;
    }

    public int getY() {
        return y;
    }

    public int getX() {
        return x;
    }
}
