/*
 * CharInfo.java
 *
 * Created on 8. jen 2005, 14:31
 *
 */
package gui.parts;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;



/**
 * Informace o znaku v Shannon-Fanove kodovani
 * @author Ondrej Hanousek
 */
public class SFCharInfo extends CharInfo {
    /**
     * sirka obdelnika
     */
    public static final int BOX_WIDTH=50;
    /**
     * vyska obdelnika
     */
    public static final int BOX_HEIGHT=40;
    /**
     * vyska obdelnika
     */
    public static final int Y_SPACING=40;
    /**
     * odsazeni zleva
     */
    public static final int X_OFFSET=10;
    /**
     * odsazeni shora
     */
    public static final int Y_OFFSET=10;
    
    private int position;
    private boolean lastBitColored=false;

    
    /**
     * vytvori informaci o znaku v Shannon-Fanove kodovani
     * @param s znak
     */
    public SFCharInfo(String s)  {
        setCharacter(s);
        position=0;
    }
    
    public void setPosition(int position){
        x=X_OFFSET;
        y=position*(Y_SPACING+BOX_HEIGHT)+Y_OFFSET;
    }
    
    public void addBit(String bit){
        setCode(getCode()+bit);
        lastBitColored=true;
    }
    
    public void resetLastBitColor(){
        lastBitColored=false;
    }
    
    /**
     * Minimalni opisujici obdelnik.
     * @return minimalni obdelnik
     */
    public Rectangle getBounds(){
        return new Rectangle(x-X_OFFSET, y-Y_OFFSET, BOX_WIDTH+X_OFFSET+2, BOX_HEIGHT+Y_OFFSET+2);
    }
    
    /**
     * Vykresleni objektu.
     * @param g1 objekt, na ktery se kresli
     */
    public void paintPart(java.awt.Graphics g1) {
        
        Graphics2D g=(Graphics2D) g1;
        g.setColor(Color.DARK_GRAY);
        Stroke oldStroke=g.getStroke();
        g.setStroke(new BasicStroke(2));
        g.drawRoundRect(x,y, BOX_WIDTH, BOX_HEIGHT, 6,6);
        g.setStroke(oldStroke);
        String popis=getCharacter().concat(" - ").concat(getCount()+"");
        Font def=g.getFont();
        g.setColor(Color.BLACK);
        g.setFont(def.deriveFont(Font.BOLD,16f));
        g.drawChars(popis.toCharArray(), 0, popis.length(), x+4, y+18);
        g.setFont(def.deriveFont(10.0f));
        if (lastBitColored) {
            g.drawChars(getCode().toCharArray(), 0, getCode().length()-1, x+5,y+34);
            FontMetrics m=g.getFontMetrics();
            int codeWidth=m.charsWidth(getCode().toCharArray(),0,getCode().length()-1);
            g.setColor(Color.RED);
            g.setFont(g.getFont().deriveFont(Font.BOLD,12f));
            g.drawChars(getCode().toCharArray(), getCode().length()-1, 1, x+5+codeWidth,y+34);
        } else {
            g.drawChars(getCode().toCharArray(), 0, getCode().length(), x+5,y+34);
        }
        // g.draw(getBounds());
        
    }
    
 
    public int getPosition() {
        return position;
    }


  
}


