/*
 * SfCharInfoRegion
 *
 * Created on 8. jen 2005, 20:18
 *
 */

package gui.parts;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.util.LinkedList;
import java.util.List;





/**
 * Zakresleni podmnoziny v Shannon Fanove kodovani.
 * @author Ondrej Hanousek
 */
public class SFCharInfoRegion extends Part {
    
    private int labelWidth=0;
    private int labelHeight=0;
    private Color color;
    private String label;
    private boolean bold=false;
    private List<Part> innerParts;
    private int width,height;
    
    /**
     * Vytvori podmnozinu v Shannon Fanove kodovani.
     * @param c barva 
     * @param label popisek
     * @param outer odkaz na nadrazenou mnozinu
     */
    public SFCharInfoRegion(Color c,String label,boolean outer) {
        innerParts=new LinkedList<Part>();
        this.color=c;
        setLabel(label);
    }
    
    public void setLabel(String label){
        this.label=label;
        Rectangle2D labelBounds=new Font("dialog", Font.BOLD, 16).getStringBounds(label,new FontRenderContext(null,false,false));
        labelWidth=(int) Math.round(labelBounds.getWidth());
        labelHeight=(int) Math.round(labelBounds.getHeight());
    }
    
    public String getLabel(){
        return label;
    }
    
    public void addInnerPart(Part p){
        innerParts.add(p);
        fireSizeChangeEvent(new SizeChange(this));
    }
    
    public void removeInnerPart(Part p){
        innerParts.remove(p);
        fireSizeChangeEvent(new SizeChange(this));
    }
    
    /**
     * Vykresleni objektu.
     * @param g1 objekt, na ktery se kresli
     */
    public void paintPart(java.awt.Graphics g1) {
        Graphics2D g=(Graphics2D) g1;
        
      //  g.draw(getInnerBounds());
        
        Color oldColor=g.getColor();
        Font oldFont=g.getFont();
        
        g.setFont(oldFont.deriveFont(Font.BOLD,16f));
        g.setColor(color);
    //    g.setStroke(new BasicStroke(1));
      //  g.draw(getBounds());

        if (bold) {
            g.setStroke(new BasicStroke(5));
        } else{
            g.setStroke(new BasicStroke(3));
        }
        Rectangle r=getInnerBounds();
        g.drawRoundRect(r.x,r.y, r.width, r.height, 8,8);
        g.drawChars(label.toCharArray(), 0, label.length(), r.x+r.width+2,
                r.y+r.height/2+(labelHeight)/2);
        //   g.translate(-x,-y);
        //   g.setColor(oldColor);
        //  g.setFont(oldFont);
    }
    
    
    /**
     * Minimalni opisujici obdelnik.
     * @return minimalni obdelnik
     */
    public Rectangle getBounds() {
       Rectangle retval=getInnerBounds();
       int pom=2;
        if (bold)  {
            pom=3;
        }
       height=retval.height+2*pom;
       width=retval.width+labelWidth+pom;
       x=retval.x-pom;
       y=retval.y-pom;
       
        return new Rectangle(x,y,width,height);
    }
    
    private Rectangle getInnerBounds(){
        x=Integer.MAX_VALUE;
        y=Integer.MAX_VALUE;
        int width=0, height=0;
        for (Part p:innerParts){
            Rectangle r=p.getBounds();
            x=Math.min(x,p.getX());
            y=Math.min(y,p.getY());
            width=Math.max(width,r.width+r.x-x);
            height=Math.max(height,r.height+r.y-y);
        }
        x-=4;
        y-=4;
        width+=8;
        height+=6;
        return new Rectangle(x, y, width, height);
    }
    public void setBold(Boolean aBoolean){
        this.bold=aBoolean;
        fireSizeChangeEvent(new SizeChange(this));
    }
    
}
