/*
 * ShCharInfo.java
 *
 * Created on 31. jen 2005, 16:20
 */

package gui.parts;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;

/**
 * Informace o znaku ve statickem huffmanove kodovani.
 * @author Ondrej Hanousek
 */
public class SHCharInfo extends CharInfo {
    /**
     * Sirka obdelniku
     */
    public static final int BOX_WIDTH=50;
    /**
     * vyska obdelnika
     */
    public static final int BOX_HEIGHT=40;
    /**
     * odsazeni zleva
     */
    public static final int X_OFFSET=15;
    /**
     * odsazeni shora
     */
    public static final int Y_OFFSET=25;
    /**
     * sirka mezi dvema objekty
     */
    public static final int X_SPACING=20;
    /**
     * odsazeni zleva
     */
    public static final int Y_SPACING=80;
    /**
     * barva fronty
     */
    public static final Color QUEUE_COLOR=Color.BLUE;
    /**
     * barva zvyrazneni
     */
    public static final Color HIGHLIGHT_COLOR=Color.RED;
    
    private boolean leaf=true;
    private SHCharInfo zero;
    private SHCharInfo one;
    private int position;
    private int width=BOX_WIDTH;
    private int height=BOX_HEIGHT;
    private boolean inQueue=false;
    private boolean selected=false;
    private boolean highlighted=false;
    private boolean newCode=true;
    private boolean h0=false;
    private boolean h1=false;
    private SHCharInfo parent;
    
    
    
    /**
     * Vytvori znak ve statickem huffmanove kodovani.
     * @param character znak
     */
    public SHCharInfo(String character) {
        setCharacter(character);
    }
    
    public SHCharInfo(int count){
        this.leaf=false;
        setCount(count);
    }
    
    
    
    public SHCharInfo(SHCharInfo one, SHCharInfo zero){
        this.leaf=false;
        setCount(one.getCount()+zero.getCount());
        setNext(one,zero);
        
        y=Math.min(one.getY(),zero.getY())-Y_SPACING;
        
        x=Math.min(zero.getX(),one.getX())+Math.abs(zero.getX()-one.getX())/2;
        one.setParent(this);
        zero.setParent(this);
    }
    
    
    private void setNext(SHCharInfo one,SHCharInfo zero){
        this.one=one;
        this.zero=zero;
    }
    
    public void setPosition(int position){
        this.position=position;
        x=X_OFFSET+position*(X_SPACING+BOX_WIDTH);
        y=Y_OFFSET;
    }
    
    public void setParent(SHCharInfo parent){
        this.parent=parent;
    }
    
    public void setInQueue(boolean isIn){
        inQueue=isIn;
        if (!isIn){
            selected=false;
        }
        fireSizeChangeEvent(new SizeChange(this));
    }
    
    public void setSelected(boolean b){
        selected=b;
        fireSizeChangeEvent(new SizeChange(this));
    }
    
    public void setHighlighted(boolean b, boolean parents){
        if (parents) highlight(b,this);
        highlighted=b;
        if (!b) newCode=false;
    }
    
    private String getCode(SHCharInfo children){
        if (parent==null) {
            if (children==zero) return "0";
            else return "1";
        }
        String pom=parent.getCode(this);
        if (children==zero) return pom+"0";
        else return pom+"1";
    }
    
    private void highlight(boolean b, SHCharInfo children){
        
        if (leaf) this.highlighted=b;
        if (children==zero) {
            h1=b;
        } else {
            h0=b;
        }
        if (parent==null) return;
        parent.highlight(b,this);
        setCode(parent.getCode(this));
        // System.out.println(code);
    }
    
    /**
     * Minimalni opisujici obdelnik.
     * @return minimalni obdelnik
     */
    public Rectangle getBounds() {
        int pom=3;
        if (inQueue || highlighted) {
            pom+=1;
            if (selected){
                pom+=1;
            }
        }
        width=BOX_WIDTH+2*pom;
        height=BOX_HEIGHT+2*pom;
        return new Rectangle(x-X_OFFSET,y-Y_OFFSET, width+X_OFFSET, height+Y_OFFSET);
    }
    
    
    /**
     * Vykresleni objektu.
     * @param g1 objekt, na ktery se kresli
     */
    public void paintPart(java.awt.Graphics g1) {
        Graphics2D g=(Graphics2D) g1;
        g.setColor(Color.DARK_GRAY);
        Stroke oldStroke=g.getStroke();
        g.setStroke(new BasicStroke(2));
        Font def=g.getFont();
        if (leaf) {
            g.drawRoundRect(x,y, BOX_WIDTH, BOX_HEIGHT, 8,8);
            String character=getCharacter();
            int count=getCount();
            String popis=character!=null?character.concat(" - ").concat(count+""):"   "+count;
            g.setColor(Color.BLACK);
            g.setFont(def.deriveFont(Font.BOLD,16f));
            int w=g.getFontMetrics().charsWidth(popis.toCharArray(), 0, popis.length());
            g.drawChars(popis.toCharArray(), 0, popis.length(), x+BOX_WIDTH/2-w/2, y+18);
            g.setFont(def.deriveFont(Font.BOLD,10f));
            if (newCode) g.setColor(HIGHLIGHT_COLOR);
            g.drawChars(getCode().toCharArray(), 0, getCode().length(), x+4, y+32);
            g.setColor(Color.BLACK);
        } else {
            g.drawOval(x,y,BOX_WIDTH,BOX_HEIGHT);
            String popis=getCount()+"";
            g.setColor(Color.BLACK);
            g.setFont(def.deriveFont(Font.BOLD,16f));
            int w=g.getFontMetrics().charsWidth(popis.toCharArray(), 0, popis.length());
            int h=g.getFontMetrics().getHeight();
            g.drawChars(popis.toCharArray(), 0, popis.length(), x+BOX_WIDTH/2-w/2, y+h+2);
            if (h0) {
                g.setColor(HIGHLIGHT_COLOR);
            }
            g.setFont(g.getFont().deriveFont(Font.PLAIN,10f));
            drawConnectingCurve(g,"1",x+BOX_WIDTH/2, y+BOX_HEIGHT, one.getX()+BOX_WIDTH/2, one.getY());
            g.setColor(Color.BLACK);
            if (h1) {
                g.setColor(HIGHLIGHT_COLOR);
            }
            g.setFont(g.getFont().deriveFont(Font.PLAIN,10f));
            drawConnectingCurve(g,"0",x+BOX_WIDTH/2, y+BOX_HEIGHT, zero.getX()+BOX_WIDTH/2, zero.getY());
            g.setColor(Color.BLACK);
            
        }
        
        g.setFont(def);
        
        
        Rectangle tmp=new Rectangle(x,y, BOX_WIDTH, BOX_HEIGHT);
        tmp.x-=3;
        tmp.y-=3;
        tmp.width+=6;
        tmp.height+=6;
        if (highlighted) {
            g.setStroke(new BasicStroke(2));
            g.setColor(HIGHLIGHT_COLOR);
            g.draw(tmp);
        }
        
        if (inQueue) {
            g.setColor(QUEUE_COLOR);
            if (selected) {
                g.setStroke(new BasicStroke(3));
            } else {
                g.setStroke(new BasicStroke(2));
            }
            if (leaf) {
                g.draw(tmp);
            } else {
                g.drawOval(tmp.x,tmp.y,tmp.width,tmp.height);
            }
        }
        g.setStroke(new BasicStroke(1));
        g.setColor(Color.BLACK);
       // g.draw(getBounds());
    }
    

    /**
     * vodorovna pozice objektu.
     * @return vodorovne poradi objektu
     */
    public int getPosition() {
        return x;
    }

}
