/*
 * Codeline.java
 *
 * Created on 8. jen 2005, 0:12
 */

package gui.utils;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;
import gui.*;


/**
 * Line of code with keywords highlight. If same word as keyword is used, should start with _ 
 * @author Ondrej Hanousek
 */
public class Codeline {
    private String codeline;
    private boolean parsed=false;
    private ArrayList<String> paragraph=new ArrayList<String>();
    private ArrayList<String> style=new ArrayList<String>();
    /**
     * Klicove slovo.
     */
    public static final String[] KEYWORDS={"begin","end","if","then","else","procedure","while","do"};
    private List<String> keyList;
    
    
    /**
     * Creates a new instance of Codeline
     * @param codeline retezec k parsovani
     */
    public Codeline(String codeline) {
        this.codeline=codeline;
        keyList=Arrays.asList(KEYWORDS);
    }
    
    private void parse(){
        if (parsed) return;
        
        for (int i=0;i<codeline.length();i++) {
            if (codeline.charAt(i)==' ') {
                paragraph.add("    ");
                style.add(AlgorithmPanel.NORMAL);
            } else {
                break;
            }
        }
        StringTokenizer tok=new StringTokenizer(codeline);
        boolean comment=false;
        while (tok.hasMoreTokens()) {
            String word=tok.nextToken();
           
            if (word.startsWith("//")) {
                comment=true;
            }
            if (comment) {
                style.add(AlgorithmPanel.COMMENT);
                continue;
            }
            if (keyList.contains(word)) {
                style.add(AlgorithmPanel.KEYWORD);
            } else {
                style.add(AlgorithmPanel.NORMAL);
            }
            if (word.startsWith("_") && word.length()>1) word=word.substring(1);
             paragraph.add(" "+word);
        }
        style.add(AlgorithmPanel.NORMAL);
        paragraph.add("\n");
        parsed=true;
    }
    
    /**
     * Kolekce slov.
     * @return jednotliva slova v kolekci
     */
    public String[] getParagraphs(){
        if (!parsed) parse();
        return paragraph.toArray(new String[paragraph.size()]);
    }
    
    /**
     * Kolekce stylu.
     * @return styly pro jednotliva slova
     */
    public String[] getStyles(){
        if (!parsed) parse();
        return style.toArray(new String[style.size()]);
    }
    
    
}
