/*
 * Stepper.java
 *
 * Created on 8. jen 2005, 11:21
 *
 */

package gui.utils;



/**
 * Trida, ktera generuje kroky.
 * @author Ondrej Hanousek
 */
public class Stepper extends Thread{
    private int time=100;
    private Steped component;
    private boolean shouldStop=false;
    /**
     * Creates a new instance of Stepper 
     */
    private  Stepper() {
        this.setPriority(Thread.MIN_PRIORITY);
    }
    
    /**
     * Vytvori generator kroku se zadanym casem a pro tridu,
     * ktera implementuje rozhrani Steped.
     * @param time cas jednoho kroku
     * @param component krokovany objekt 
     */
    public Stepper(int time, Steped component ){
        this();
        this.time=time;
        this.component=component;
    }
    
    /**
     * Vytvori krokovac pro objekt, ktery implementuje rozhrani
     * Steped.
     * @param component krokovany objekt
     */
    public Stepper(Steped component){
        this();
        this.component=component;
    }
    
    /**
     * Spusti generovani kroku az do okamziku, nez 
     * je vraceno -1 a nebo nez je zavolano steppingEnded().
     */
    public void run() {
        
        int ret=0;
        do {
            if (shouldStop) return;
            ret=component.doNextStep();
            try {
                this.sleep(time);
            } catch (InterruptedException ex) {
                System.err.println("Internal error - stepper waked up");
            }
        }
        while (ret!=-1);
        component.steppingEnded();
    }
    
    /**
     * Ukonci automaticke krokovani.
     */
    public void endStepping(){
        shouldStop=true;
    }
    
}
