/*
 * FGKOutputStream.java
 *
 * Created on 3. listopad 2005, 10:28
 */

package streams;

import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.SortedSet;
import java.util.TreeSet;
import pradny.BitOutputStream;
import streams.utils.ByteInfoFGK;
import streams.utils.ByteInfoFGKOrderComparator;



/**
 * Trida, realizujici adaptivni Huffmanovo kodovani.
 * @author Ondrej Hanousek
 */
public class FGKOutputStream extends FilterOutputStream {
    private ByteInfoFGK root, zeroNode;
    private ByteInfoFGK[] byteTable=new ByteInfoFGK[256];
    private TreeSet<ByteInfoFGK> allNodes=new TreeSet<ByteInfoFGK>(new ByteInfoFGKOrderComparator());
    private BitOutputStream out;
    private int inc=0;
    /**
     * Vytvori adaptivni Huffmanuv koder.
     * @param os podrizeny stream, do ktereho se bude zapisovat.
     */
    public FGKOutputStream(OutputStream os) {
        super(os);
        out=new BitOutputStream(os);
        zeroNode=new ByteInfoFGK();
        root=zeroNode;
        allNodes.add(root);
    }
    
    
    /**
     * Zapise data do streamu, upravi strom.
     * @param b data k zapsani
     * @throws java.io.IOException pri chybe se zapisem nebo kodovanim
     */
    public synchronized void write(int b) throws java.io.IOException {
        //System.out.println(inc++);
        ByteInfoFGK incNode;
        
        if (byteTable[b]==null) { //precteni dosud neprectenoho byte
           out.write(zeroNode.getCode());
            //out.write((byte) b);
           out.write(byteToBoolArray((byte) b));
            
 //           ByteInfoFGK oldParent=zeroNode.getParent();
            ByteInfoFGK newByte=new ByteInfoFGK(b);
            ByteInfoFGK newZero=new ByteInfoFGK();
            byteTable[b]=newByte;
            
            // System.out.println(Arrays.toString(zeroNode.getCode()) + " "+ newByte.getData());
          
            int order=zeroNode.getOrder();
            newByte.setOrder(order-1);
            newZero.setOrder(order-2);
            zeroNode.setChildren(newZero, newByte);
            
            allNodes.add(newByte);
            allNodes.add(newZero);
            incNode=newByte;
            
            zeroNode=newZero;

            //   System.out.println(newByte);
        } else { //Zvysovani cetnosti
            incNode=byteTable[b];
            out.write(incNode.getCode());
            
        }
        updateTree(allNodes, incNode);
     //   root.updateChildrenCode();
    //   System.out.println(allNodes);
    }
    
    /**
     * aktualizuje strom - hlida sourozeneckou vlastnost
     * @param nodes uzly setridene podle poradi
     * @param node uzel, u ktere mohlo dojit k naruseni sourozenecke
     * vlastnosti
     */
    public static void updateTree(SortedSet<ByteInfoFGK> nodes, ByteInfoFGK node ){
        
        while (!node.isRoot()) {
            long count=node.getCount();
            SortedSet<ByteInfoFGK> greater=nodes.tailSet(node);
            ByteInfoFGK tmp=node;
            for (ByteInfoFGK g:greater){
                // System.out.println(g);
                if (g.getCount()==count) {
                    tmp=g;
                }
            }
            if (tmp!=node && tmp!=node.getParent()) {
                nodes.remove(tmp);
                nodes.remove(node);
                swapNodes(tmp,node);
                node.increment();
                nodes.add(tmp);
                nodes.add(node);
                
          //      break;
            } else {
                node.increment();
            }
            node=node.getParent();
        }
        node.increment();
        
    }
    
    private static void swapNodes(ByteInfoFGK node1,ByteInfoFGK node2){
        ByteInfoFGK p1=node1.getParent();
        ByteInfoFGK p2=node2.getParent();
        int pomOrder=node1.getOrder();
        node1.setOrder(node2.getOrder());
        node2.setOrder(pomOrder);
        p1.changeChildrenNode(node1, node2);
        p2.changeChildrenNode(node2,node1);
        p1.updateChildrenCode();
        p2.updateChildrenCode();
    }
    
    /**
     * uzavre zapisovy stream
     * @throws java.io.IOException chyba uzavirani
     */
    public void close() throws IOException {
        out.close();
        super.close();
    }
    
 
    

       private static boolean[] byteToBoolArray(byte b){
        boolean[] ret=new boolean[8];
        int mask=1;
        for (int i=7;i>=0;i--){
            ret[i]=((b & mask) > 0);
            mask=mask <<1;
        }
        
        //      System.out.println(Arrays.toString(ret)+" "+b);
        return ret;
    }
    
}
