/*
 * SFInputStream.java
 *
 * Created on 1. jen 2005, 14:17
 *
 * Shannon Fano decompresion.
 */

package streams;

import hanus.FibonacciDecoder;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import pradny.BitInputStream;
import pradny.EOSException;
import pradny.UnsupportedInputException;
import streams.utils.ByteInfo;
import streams.utils.ByteInfoCodeComparator;
import streams.utils.ByteInfoCountComparator;

/**
 * Dekoder streamu zakodovaneho Shannon-Fanovym kodovanim.
 * Predpoklada ulozeni statistickeho modelu zacatku souboru 
 * Fibonacciho kodem radu 2.
 * @author Ondrej Hanousek
 */
public class SFInputStream extends FilterInputStream {

    private BitInputStream bis;
    private ByteInfo[] byteTable=new ByteInfo[256];
    private long size=0;
    private boolean[] code=new boolean[256];
    private int used=256;
    private int lastOffset=0;
    
    /**
     * Vytvori objekt pro dekodovani streamu zakodovanem Shanon-Fanovym
     * kodovanim. Nacte ze vstupu cetnosti bajtu a vytvori kodovy strom
     * @param is vstupni stream
     * @throws pradny.UnsupportedInputException chybny vstup
     * @throws pradny.EOSException chyba cteni
     */
    public SFInputStream(InputStream is) throws UnsupportedInputException, EOSException  {
        super(is);
        bis=new BitInputStream(is);
        FibonacciDecoder dec=new FibonacciDecoder(bis);
     
        for (int i=0;i<byteTable.length;i++){
            long pom=dec.decodeLong()-1;
            size+=pom;
            byteTable[i]=new ByteInfo(i,pom);
        }
        Arrays.sort(byteTable,new ByteInfoCountComparator());
        SFOutputStream.doCodes(byteTable, 0, byteTable.length, size);
       // System.out.println(Arrays.toString(byteTable));
        Arrays.sort(byteTable,new ByteInfoCodeComparator());
    }
    
        /**
     * Vraci data nactena a dekodovana ze vstupniho proudu.
     * @return dalsi byte nebo <pre>-1</pre> pri konci vstupu
     * @exception IOException pri I/O chybe
     * @see java.io.FilterInputStream#in
     */
    public int read() throws IOException {
 //       int pos=0;
        int found;
        if (size==0) { return -1; }
        size --;
        int i=0;
        
        int toRead=Math.min(used,bis.bitsAvailable());
        
        boolean[] temp=new boolean[toRead];
        
        try {
            if (used<bis.bitsAvailable()) {
                temp=bis.readBits(toRead);
            } else {
                for (i=0;i<toRead;i++) {
                    temp[i]=bis.readBit();
                }
            }
        } catch (EOSException e) {
            //System.out.println("Nenalezeno, "+e.AvailableBits()+" "+i);
        }
        
        System.arraycopy(code, used, code, 0, 256-used);
        System.arraycopy(temp, 0, code, 256-used, toRead);
        
        ByteInfo pom=new ByteInfo(code);
        found=Arrays.binarySearch(byteTable,pom, new ByteInfoCodeComparator());
        
        if (found<0)  { found=(-found)-2; }
        used=byteTable[found].getCodeLength();
        return (byteTable[found].getData() & 0xff);
    }

    /**
     * Uzavre stream.
     * @throws java.io.IOException pri I/O chybe
     */
    public void close() throws IOException {
        bis.close();
        super.close();
    }
 
    
}
