/*
 * SHInputStream.java
 *
 * Created on 26. jen 2005, 1:01
 *
 */

package streams;

import hanus.FibonacciDecoder;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import pradny.BitInputStream;
import pradny.EOSException;
import pradny.UnsupportedInputException;
import streams.utils.ByteInfoCodeComparator;
import streams.utils.ByteInfoCountComparator;
import streams.utils.ByteInfoSH;


/**
 * Dekoder streamu zakodovaneho statickym Huffmanovym kodovanim.
 * Ocekava ulozeni cetnosti jednotlivych bajtu Fibonacciho kodem
 * radu 2 a data.
 * @author Ondrej Hanousek
 */
public class SHInputStream extends FilterInputStream {
    
    private BitInputStream bis;
    private ByteInfoSH[] byteTable=new ByteInfoSH[256];
    private long size=0;
    private boolean[] code=new boolean[256];
    private int used=256;
    private int lastOffset=0;
    private ByteInfoSH root;
    
    /**
     * Vytvori dekoder, ze vstupu nacte cetnosti bajtu a
     * vytvori kodovy strom.
     * @param is vstupni stream
     * @throws pradny.UnsupportedInputException spatny format vstupu.
     * @throws pradny.EOSException chyba vstupu.
     */
    public SHInputStream(InputStream is) throws UnsupportedInputException, EOSException  {
        super(is);
        bis=new BitInputStream(is);
        FibonacciDecoder dec=new FibonacciDecoder(bis);
        
        for (int i=0;i<256;i++){
            long pom=dec.decodeLong()-1;
            size+=pom;
            byteTable[i]=new ByteInfoSH(i,pom);
        }
        Arrays.sort(byteTable,new ByteInfoCountComparator());
        root=SHOutputStream.buildHuffmanTree(byteTable);
        Arrays.sort(byteTable,new ByteInfoCodeComparator());
    }
    
    
    /**
     * cteni dat ze vstupniho proudu a jejich dekodovani.
     * @return dekodovana data nebo -1 na konci streamu
     * @exception IOException pri I/O chybe
     * @see java.io.FilterInputStream#in
     */
    public int read() throws IOException {
        ByteInfoSH current=root;
        if (root.isLeaf()) {
            size--;
            if (size>=0) { return root.getData(); }
            else { return -1; }
        }
        try {
            while (!current.isLeaf()) {
                current=current.getNext(bis.readBit());
            }
        } catch (EOSException e) {
            return -1;
        } catch (NullPointerException ex){
            return -1;
        }
        
        return (current.getData()  & 0xff);
    }
    
    /**
     * uzavre dekodovaci stream.
     * @throws java.io.IOException pri I/O chybe
     */
    public void close() throws IOException {
        bis.close();
        super.close();
    }
    
}
