/*
 * ByteInfo.java
 *
 * Created on 1. jen 2005, 14:10
 *
 */

package streams.utils;


/**
 * Trida s informacemi o bajtu - o hodnote, kodu a pocitadle.
 * @author Ondrej Hanousek
 */
public class ByteInfo {
    
    private byte data=0;
    private long count=0;
    protected int size=0;
    protected boolean[] codeArray=new boolean[0];
    
    /**
     * Vytvori byteInfo pro dany bajt
     * @param data bajt
     */
    public ByteInfo(int data){
        this.data=(byte)(data & 0xff);
    }
    
    /**
     * vytvori byteInfo pro dany bajt s prednastavenym pocitadlem
     * @param data bajt
     * @param count pocitadlo
     */
    public ByteInfo(int data, long count){
        this.data=(byte)(data & 0xff);
        this.count=count;
    }
    
    /**
     * Vytvori byteInfo se zadanym kodem
     * @param code kod
     */
    public ByteInfo(boolean[] code){
        this.data=0;
        this.codeArray=code;
        this.size=code.length;
    }
    
    /**
     * zvysi pocitadlo
     */
    public void increment(){
        count++;
    }
    
    /**
     * Vypise informace i bytu.
     * @return informace o bytu - cetnosti a kodu.
     */
    public String toString(){
        return (data  & 0xff)+": "+count+" "+getCodeString();
    }
    
    /**
     * Delka kodu, kterym je bajt zakodovan.
     * @return delka kodu
     */
    public int getCodeLength() {
        return size;
    }
    
    /**
     * Kod ve forme bitoveho pole
     * @return pole s kodem
     */
    public boolean[] getCodeBits() {
        return codeArray;
    }
    
    
    /**
     * Kod ve forme retezce
     * @return retezec s kodem
     */
    public String getCodeString() {
        StringBuffer out=new StringBuffer(size);
        if (size==0) return "_";
        for (int i=0;i<size;i++) {
            out.append(codeArray[i]?1:0);
        }
        return  out.toString();
    }
    /**
     * nastaveni pocitadla
     * @param count novy stav pocitadla
     */
    public void setCount(long count){
        this.count=count;
    }
    /**
     * Vraci aktalni pocitadlo.
     * @return pocet
     */
    public long getCount(){
        return count;
    }
    
    /**
     * Prirazeny bajt.
     * @return prislusny bajt.
     */
    public byte getData(){
        return data;
    }
    
    /**
     * Nastaveni/zmena aktualniho bajtu.
     * @param data novy bajt
     */
    public void setData(int data){
          this.data=(byte)(data & 0xff);
    }
    
    /**
     * Prida bit ke kodu (na konec). Potrebuje opltimalizaci pro rychlejsi funkci.
     * @param bit dalsi bit ke kodu.
     */
    public void addBit(boolean bit){
       boolean[] tmp=new boolean[size+1];
       System.arraycopy(codeArray,0,tmp,0, size);
        tmp[size]=bit;
        size++;
        codeArray=tmp;
    }
}

