/*
 * ByteInfoFGK.java
 *
 * Created on 3. listopad 2005, 19:23
 *
 
 */

package streams.utils;

import java.util.Arrays;


/**
 * Trida pro informace o bajtu pro adaptivnim huffmanove kodovani.
 * @author Ondrej Hanousek
 */
public class ByteInfoFGK extends ByteInfo {
    private ByteInfoFGK zero,one,parent;
    private int order;
    
    private boolean[] code=new boolean[0];
    private boolean isZero=false;
    
    
    
    /** Creates a ZeroNode */
    public ByteInfoFGK() {
        super(-1);
        
        order=100;
        isZero=true;
    }
    
    /**
     * Vytvori ByteInfo pro dana data.
     * @param data bajt
     */
    public ByteInfoFGK(int data){
        
        super(data);
    }
    
    /**
     * Vytvori nadrizeny uzel pro dane dva uzly.
     * @param zero levy potomek
     * @param one pravy potomek
     */
    public ByteInfoFGK(ByteInfoFGK zero, ByteInfoFGK one){
        super(-1,zero.getCount()+one.getCount());
        this.zero=zero;
        this.one=one;
        one.setParent(this);
        zero.setParent(this);
        
    }
    
    /**
     * nastavi rodice pro uzel
     * @param parent novy rodic
     */
    public void setParent(ByteInfoFGK parent){
        this.parent=parent;
    }
    
    /**
     * Nastavi potomky pro uzel
     * @param zero levy potomek
     * @param one pravy potomek
     */
    public void setChildren(ByteInfoFGK zero, ByteInfoFGK one){
        this.zero=zero;
        this.one=one;
        zero.setParent(this);
        one.setParent(this);
        setData(-1);
        isZero=false;
        updateChildrenCode();
    }
    
    /**
     * Vraci rodicovsky uzel.
     * @return rodic
     */
    public ByteInfoFGK getParent(){
        return parent;
    }
    
    /**
     * Test, zda je vrcholovym uzlem.
     * @return true, prokud nema rodice
     */
    public boolean isRoot(){
        return parent==null;
    }
    
    
    /**
     * Nastavi ocislovani
     * @param order nove ocislovani
     */
    public void setOrder(int order){
        this.order=order;
        
    }
    
    /**
     * Vrati ocislovani
     * @return ocislovani uzlu
     */
    public int getOrder(){
        return order;
    }
    
    /**
     * Zameni potomka zadaneho jako currNode za noveho.
     * @param currNode soucasny potomek
     * @param newNode novy potomek
     */
    public void changeChildrenNode(ByteInfoFGK currNode, ByteInfoFGK newNode){
        long count=getCount();
        if (zero==currNode) zero=newNode;
        else
            if (one==currNode) one=newNode;
            else System.err.println("CHYBA!!!");
            
        newNode.setParent(this);
        updateChildrenCode();
    }
    
    /**
     * vypise informace o bajtu.
     * @return vypis
     */
    public String toString(){
        byte pom=getData();
        if (isZero()) return "Z " + getOrder()+Arrays.toString(getCode());
        if (!isLeaf()) {
            try {
                return order+" "+getCount()+" "+isLeaf()+" 0>"+zero.getOrder()+" 1>"+one.getOrder()+Arrays.toString(getCode());
            } catch (Exception e) {
                System.out.println("S");
            }
            
        }
        return ((char) pom + " " + order+" "+getCount()+Arrays.toString(getCode()));
    }
    
    /**
     * Porovnani podle poradi (order)
     * @param obj dalsi FGKCharInfo
     * @return rozdil poradi
     */
    public boolean equals(Object obj){
        ByteInfoFGK b1=(ByteInfoFGK) obj;
        return b1.order==order;
    }
    
    /**
     * Vraci leveho potomka
     * @return levy potomek
     */
    public ByteInfoFGK getZero(){
        return zero;
    }
    
    /**
     * Metoda pro zjisteni praveho potomka
     * @return pravy potomek
     */
    public ByteInfoFGK getOne(){
        return one;
    }
    /**
     * Zjisteni, zda je uzel levym potomkem rodice.
     * @return true kdy uzel je levym potomkem rodice
     */
    public boolean isZero(){
        return (isZero);
    }
    
    
    /**
     * Metoda pro ziskani kodu.
     * @return aktualni kod
     */
    public boolean[] getCode(){
        return code;
    }
    
    /**
     * Metoda pro nastaveni kodu
     * @param newCode novy kod
     */
    public void setCode(boolean[] newCode){
        this.code=newCode;
        //   System.out.println(zzero.isLeaf());
        
    }
    
    /**
     * Metoda pro zjisteni, zda ma uzel potomky.
     * @return true kdyz uzel nema potomky
     */
    public boolean isLeaf(){
        return zero==null;
    }
    
    /**
     * metoda pro aktualizaci kodu.
     */
    public void updateChildrenCode(){
        if (isLeaf()) return;
        boolean[] code0=new boolean[code.length+1];
        boolean[] code1=new boolean[code.length+1];
        System.arraycopy(code, 0, code0, 0, code.length);
        System.arraycopy(code, 0, code1, 0, code.length);
        code0[code.length]=false;
        zero.setCode(code0);
        code1[code.length]=true;
        one.setCode(code1);
        zero.updateChildrenCode();
        one.updateChildrenCode();
    }
    
    
}
