/*
 * ByteInfoNode.java
 *
 * Created on 25. jen 2005, 22:56
 *
 */

package streams.utils;


/**
 * Trida pro udrzovani informaci o bajtu pri statickem huffmanove kodovani.
 * @author Ondrej Hanousek
 */
public class ByteInfoSH extends ByteInfo {
    private ByteInfoSH zero;
    private ByteInfoSH one;
    boolean leaf=false;
    
    /**
     * Vytvori uzel se dvema potomky.
     * @param zero levy potomek
     * @param one pravy potomek
     */
    public ByteInfoSH(ByteInfoSH zero, ByteInfoSH one){
        super(0);
        setCount(zero.getCount()+one.getCount());
        setSubnodes(zero, one);
        
    }
    
    /**
     * Vytvori informaci o bajtu se zadanym pocitadlem a bajtem.
     * @param data bajt
     * @param count pocitadlo
     */
    public ByteInfoSH(int data,long count) {
        super(data,count);
    }
    
    /**
     * Vytvori informaci o bajtu se zadanym bajtem.
     * @param data bajt
     */
    public ByteInfoSH(int data) {
        super(data);
    }
    
    /**
     * Vytvori ByteInfo pro dany kod.
     * @param bits kod
     */
    public ByteInfoSH(boolean[] bits){
        super(bits);
    }
    /**
     * Nastaveni, zda muze mit potomky.
     * @param b true kdyz nesmi mit potomky
     */
    public void setLeaf(boolean b){
        this.leaf=b;
    }
    
    /**
     * Test, jestli ma potomky.
     * @return true kdyz nema potomky
     */
    public boolean isLeaf(){
        return leaf;
    }
    
    /**
     * Dalsi uzel pri prochazeni kodovym stromem.
     * @param bit bit pro vyber leveho nebo praveho potomka
     * @return dalsi uzel
     */
    public ByteInfoSH getNext(boolean bit) {
        if (leaf) return null;
        if (bit) return one;
        else return zero;
    }
    
    /**
     * nastaveni potomku
     * @param zero levy potomek
     * @param one pravy potomek
     */
    public void setSubnodes(ByteInfoSH zero, ByteInfoSH one){
        this.zero=zero;
        this.zero.addBit(false);
        this.one=one;
        this.one.addBit(true);
    }
    
    /**
     * Ziskani kodu uzlu
     * @return kod uzlu
     */
    public boolean[] getCodeBits(){
        boolean[] pom=super.getCodeBits();
        boolean[] out=new boolean[pom.length];
        
        for (int i=0;i<pom.length;i++){
            out[i]=pom[pom.length-i-1];
        }
        return out;
    }
    
    /**
     * pridani bitu ke kodu uzlu (na zacatek)
     * @param bit bit, ktery se ma pridat
     */
    public void addBit(boolean bit) {
        super.addBit(bit);
        if (!isLeaf()) {
            zero.addBit(bit);
            one.addBit(bit);
        }
    }
    

}
